/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
 #include "extern.h"

/* --------------------------
   | Connect via connection |
   -------------------------- */
void AppConnect(int *msg)
{
DEBUG();
get_connection(*(char **)(msg + 4));
copy_connection(0);

if (!check_connection(0x4174))
	return;

connect_now();
}

/* -------------------------
   | Connect via parameter |
   ------------------------- */
void AppConnectS(int *msg)
{
char *host, *user, *password;

DEBUG();
host = *(char **)(msg + 4);
user = host + strlen(host) + 1;
password = user + strlen(user) + 1;

strcpy(act_conndef.host, host);
strcpy(act_conndef.user, user);
strcpy(act_conndef.password, password);

if (!check_connection(0x4174))
	return;

connect_now();
}

/* -------------------
   | Cd to directory |
   ------------------- */
void AppCd(int *msg)
{
char *ptr;

DEBUG();
ptr = *(char **)(msg + 4);
if (ptr)
	ftp_setcwd(ftp_socket, ptr);
}

/* ------------
   | Get file |
   ------------ */
void AppGet(int *msg)
{
char *rcvbuf;
long a, received;
int bufsize;
char *remote, *local;

DEBUG();
remote = *(char **)(msg + 4);
local = remote + strlen(remote) + 1;

if (!*local)
	if (strchr(remote, '\\'))
		local = strrchr(remote, '\\') + 1;
	else
		local = remote;
		
received = 0;

if (ftp_retrieve(ftp_socket, remote, 0L))
	{
	SetInfo(recvstring);

	rcvbuf = (char *)malloc(ftp_opts.in_bufsize);

	transmission_start = clock();
 	open_cpyinfo(0, remote);
 
	a = Fcreate(local, 0);
	
	if (a < 0)
		{
		form_alert(1, create_error);
		close_cpyinfo();
		free(rcvbuf);
		ftp_abort(ftp_socket);
		SetInfo(aborted);
		return;
		}
					
	while((bufsize = ftp_getblock(ftp_socket, ftp_opts.in_bufsize, rcvbuf)) > 0)
		{
		Fwrite((int)a, bufsize, rcvbuf);

		received += bufsize;
		set_cpyinfo(received, 1);
		}
	close_cpyinfo();
	free(rcvbuf);
	
	Fclose((int)a);
	SetInfo(done);

	/* ok */
	}
/* Error */
}

/* ------------
   | Put file |
   ------------ */
void AppPut(int *msg)
{
char *local, *remote;
char *sndbuf;
int bufsize;
long sent, a, totalsize;

DEBUG();
local = *(char **)(msg + 4);
remote = local + strlen(local) + 1;

if (!*remote)
	if (strchr(local, '\\'))
		remote = strrchr(local, '\\') + 1;
	else
		remote = local;

if (ftp_store(ftp_socket, remote, 0L))
	{
	SetInfo(sendstring);

	sndbuf = (char *)malloc(ftp_opts.out_bufsize);
	sent = 0;

	transmission_start = clock();
 	open_cpyinfo(1, remote);
		
	a = Fopen(local, FO_READ);
	totalsize = Fseek(0L, (int)a, 2);
	Fseek(0L, (int)a, 0);
	
	while((bufsize = (int)Fread((int)a, ftp_opts.out_bufsize, sndbuf)) > 0)
		{
		if (ftp_putblock(ftp_socket, bufsize, sndbuf) != bufsize)
			break;
			
		sent += bufsize;
		
		set_cpyinfo(sent, totalsize);
		}

	if (ftp_putblock(ftp_socket, 0, NULL) == -1)
		{}

	close_cpyinfo();

	free(sndbuf);
	Fclose((int)a);
	SetInfo(done);

	/* Okay */
	}
/* Error */
}

/* ------------
   | Set type |
   ------------ */
void AppType(int *msg)
{
DEBUG();
ftp_binary(ftp_socket, msg[4]);
}

/* --------------------
   | Close connection |
   -------------------- */
void AppClose(int *msg)
{
DEBUG();
disconnect();
if (msg[4] == 1)
	exit_flg = TRUE;
}
