/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static int help_ids[] = { DOCONN, CANCCONN, C_DEL, C_SAVE,
													COPTS1, COPTS2, COPTS3,						/* 0-6 */
													CONNNAME, CSELCONN, SRV_NAME,
												 	ANOLOGIN, USERNAME, PASSWORD,
													ACCNAME, SAV_PWD, 								/* 7-14 */
													PORTNUMB,	USEPASV, TIMEOUT, 
													SHOWDOTS,													/* 15-18 */
													FILESMIX, FILESUP, FILESLOW,
													LOCALDIR, REMOTDIR,								/* 19-23 */
												
												0};
												
static int helpinitialized = FALSE;

/* ------------------------
   | Connection selection |
   ------------------------ */
int drop_connect(void)
{
char **strings;
char *buffer, *ptr;
int i, ret, x, y;
OBJECT *obj;

DEBUG();
obj = connect_dial + (connect_tabs[0] + CONNNAME);
objc_offset(connect_dial, connect_tabs[0] + CONNNAME, &x, &y);

strings = (char **)malloc(sizeof(char *) * (conn_cnt + 1));
if (!strings)
	return FALSE;
	
ptr = buffer = (char *)malloc(conn_cnt * 29);
if (!buffer)
	{
	free(strings);
	return FALSE;
	}

for (i = 0; i < conn_cnt; i++)
	{
	strings[i] = ptr;
	strcpy(ptr, conn_defs[i].connection);
	ptr += 29;
	}
strings[i] = NULL;

ret = free_droplist(x, y, obj->ob_width, strings);
if (ret != -1)
	strcpy(obj->ob_spec.tedinfo->te_ptext, strings[ret]);

free(buffer);	
free(strings);

return ret != -1;
}

/* -------------------
   | Set option page |
   ------------------- */
void SetConnPage(OBJECT *tree, int page, int obj)
{
static int tabids[] = {COPTS3, COPTS2, COPTS1};
int i;

DEBUG();
if (tree[obj].ob_flags & SELECTABLE)
	{
	BindTab(tree, COPTBKG, page);
	redraw_object(tree, COPTBKG);
	
	for (i = 0; i < 3; i++)
		{
		tree[tabids[i]].ob_flags |= SELECTABLE|EXIT;
		SetSelect(tree, tabids[i], obj == tabids[i]);
		redraw_object(tree, tabids[i]);
		}
	tree[obj].ob_flags &= ~(SELECTABLE|EXIT);
	redraw_object(tree, obj);
	}
}

/* --------------------
   | Connect a server |
   -------------------- */
void connect_to(void)
{
int obj;

DEBUG();
copy_connection(0);

if (!helpinitialized)
	{
	int i;

	for (i =  7; i <= 14; i++)	help_ids[i] += connect_tabs[0];		
	for (i = 15; i <= 17; i++)	help_ids[i] += connect_tabs[1];		
	for (i = 18; i <= 23; i++)	help_ids[i] += connect_tabs[2];		
	
	helpinitialized = TRUE;
	}

SetConnPage(connect_dial, connect_tabs[0], COPTS1);
open_kinddialog(connect_dial, connect_dial[CONN_TITLE].ob_spec.free_string, iconified, SMALLER|NAME|MOVER|CLOSER);
dlgset_help_info(connect_dial, helpfile, help_ids, "CONN_");
do
	{
	obj = do_dialog(connect_dial, 0, app_callback);
	switch(obj)
		{
		case COPTS1:		SetConnPage(connect_dial, connect_tabs[0], COPTS1);	break;
		case COPTS2:		SetConnPage(connect_dial, connect_tabs[1], COPTS2);	break;
		case COPTS3:		SetConnPage(connect_dial, connect_tabs[2], COPTS3);	break;
		
		case C_SAVE:
			copy_connection(TRUE);
			save_connection();
			break;
			
		case C_DEL:
			copy_connection(TRUE);
			delete_connection();
			memcpy(&act_conndef, &conndefault, sizeof(CONN_INF));
			copy_connection(0);
			redraw_dialog(connect_dial);
			break;
			
		default:
			if (IsSelected(connect_dial, COPTS1))	/* First Connect Page active ? */
				{
				switch(obj - connect_tabs[0])
					{
					case CSELCONN:
						if (drop_connect())
							{
							get_connection(connect_dial[connect_tabs[0] + CONNNAME].ob_spec.tedinfo->te_ptext);
							copy_connection(0);
							redraw_object(connect_dial, connect_tabs[0]);
							}
						break;
					}
				}
		}
	}while(obj != DOCONN && obj != CANCCONN && obj != (0x2000|WM_CLOSED));
close_dialog(connect_dial, TRUE);

copy_connection(obj == DOCONN);

if (obj == DOCONN)
	{
	if (act_conndef.init_dir[0])
		{
		strcpy(local_dir, act_conndef.init_dir);
		GetLocaldir();
		}
		
	if (!check_connection(0x4174))
		return;

	connect_now();
	set_buttonstate();
	}
}

/* ---------------------------------
   | Connect with current settings |
   --------------------------------- */
void connect_now(void)
{
char *user, *password;

DEBUG();
SetInfo(connecting);
ftp_socket = ftp_connect(act_conndef.host, act_conndef.port_number);
if (!ftp_socket)
	form_alert(1, connect_unable);	
else
	{
	SetInfo(userauth);
	
	if (act_conndef.anonymous)
		{
		user = "anonymous";
		password = ftp_opts.ano_email;
		}
	else
		{
		user = act_conndef.user;
		password = act_conndef.password;
		}
		
	if (!ftp_login(ftp_socket, user, password))
		{
		form_alert(1, login_error);
		disconnect();
		return;
		}
			
	SetInfo(connected);
	
	if (act_conndef.remote_dir[0])
		ftp_setcwd(ftp_socket, act_conndef.remote_dir);
		
	ftp_getcwd(ftp_socket, ftp_remote);
	ReadDirectory();
				
	SetInfo(done);

	EnableObj(maindial, SEND); 			redraw_object(maindial, SEND);
	EnableObj(maindial, RECEIVE); 	redraw_object(maindial, RECEIVE);
	EnableObj(maindial, FDISCONN); 	redraw_object(maindial, FDISCONN);
	DisableObj(maindial, FCONNECT); redraw_object(maindial, FCONNECT);
	}
}

/* --------------
   | Disconnect |
   -------------- */
void disconnect(void)
{
DEBUG();
if (ftp_socket)
	{
	SetInfo(closeconnection);
	ftp_close(ftp_socket);
	ftp_socket = 0;
	SetInfo("\0");

	DisableObj(maindial, SEND); 		redraw_object(maindial, SEND);
	DisableObj(maindial, RECEIVE); 	redraw_object(maindial, RECEIVE);
	DisableObj(maindial, FDISCONN); redraw_object(maindial, FDISCONN);
	EnableObj(maindial, FCONNECT); 	redraw_object(maindial, FCONNECT);
	set_buttonstate();
	}
}

