/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
 #include "extern.h"

/* ---------------------
   | Copy info handler |
   --------------------- */
static void cdecl closecpy_handler (OBJECT *tree, int obj)
{
DEBUG();
close_dialog(tree, TRUE);
}

/* ----------------------------
   | Is Copy info still open? |
   ---------------------------- */
int is_cpyinfoOpen(void)
{
DEBUG();
return is_open(cpyinfo);
}

/* ----------------------------
   | Display copy information |
   ---------------------------- */
void open_cpyinfo(int op, char *fname)
{
DEBUG();
if (op)
	cpyinfo[OPINFO].ob_spec.free_string = sendstring;
else
	cpyinfo[OPINFO].ob_spec.free_string = recvstring;

cpyinfo[OPSLIDER].ob_width = 0;
strncpy(cpyinfo[FNAME_PH].ob_spec.free_string, fname, 33);
cpyinfo[BYTE_PH].ob_spec.tedinfo->te_ptext[0] = 0;
cpyinfo[REM_PH].ob_spec.tedinfo->te_ptext[0] = 0;
cpyinfo[BPS_PH].ob_spec.tedinfo->te_ptext[0] = 0;

open_dialog(cpyinfo, cpyinfo[COPY_TITLE].ob_spec.free_string, iconified);
set_active(cpyinfo, closecpy_handler, TRUE, 0);
}

/* ------------------------
   | Set copy information |
   ------------------------ */
void set_cpyinfo(long currsize, long totalsize)
{
int hour, min, sec;
long remaining, secs;

DEBUG();
sprintf(cpyinfo[BYTE_PH].ob_spec.tedinfo->te_ptext,
				byte_info, totalsize, currsize);

if (!ftp_opts.bps_mode)
	sprintf(cpyinfo[BPS_PH].ob_spec.tedinfo->te_ptext,
					bps_info, currsize * 8 * CLK_TCK / (clock() - transmission_start));
else
	sprintf(cpyinfo[BPS_PH].ob_spec.tedinfo->te_ptext,
					byt_info, currsize * CLK_TCK / (clock() - transmission_start));

if (totalsize)
	{
	cpyinfo[OPSLIDER].ob_width = (int)(cpyinfo[OPSLDBKG].ob_width * currsize / 
															 totalsize);
	
	remaining = totalsize - currsize;
	secs = remaining / (currsize * CLK_TCK / (clock() - transmission_start));
	
	hour = (int)(secs / 3600);
	secs %= 3600;
	min = (int)(secs / 60);
	secs %= 60;
	sec = (int)secs;
	sprintf(cpyinfo[REM_PH].ob_spec.tedinfo->te_ptext,
					"%02d:%02d:%02d", hour, min, sec);
	}
else
	strcpy(cpyinfo[REM_PH].ob_spec.tedinfo->te_ptext, unknown);
	
redraw_object(cpyinfo, BYTE_PH);
redraw_object(cpyinfo, BPS_PH);
redraw_object(cpyinfo, REM_PH);
redraw_object(cpyinfo, OPSLIDER);
}

/* ----------------------------
   | Close copy info dialogue |
   ---------------------------- */
void close_cpyinfo(void)
{
DEBUG();
close_dialog(cpyinfo, TRUE);
}