/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
 #include "extern.h"

/* -----------------
   | Bind tab page |
   ----------------- */
void BindTab(OBJECT *tree1, int obj1, int new_obj)
{
int i;

DEBUG();
if (tree1[obj1].ob_head != -1)
	{
	i = tree1[obj1].ob_head;
	for (;;)
		{
		if (((tree1[i].ob_type & 0xFF) == G_FTEXT || 
				 (tree1[i].ob_type & 0xFF) == G_FBOXTEXT) &&
				 (tree1[i].ob_flags & EDITABLE))
			{
			tree1[i].ob_type |= 0x80;
			tree1[i].ob_flags &= ~EDITABLE;
			}

		if (tree1[i].ob_flags & (0x8000|LASTOB))
			break;
			
		i++;
		}

	objc_delete(tree1, tree1[obj1].ob_head);
	}
	
i = new_obj;
for(;;)
	{
	if ((tree1[i].ob_type & 0xFF) == (G_FTEXT|0x80) || 
			(tree1[i].ob_type & 0xFF) == (G_FBOXTEXT|0x80))
		{
		tree1[i].ob_type &= ~0x80;
		tree1[i].ob_flags |= EDITABLE;
		}
	if (tree1[i].ob_flags & (0x8000|LASTOB))
		break;
	i++;
	}
	
objc_add(tree1, obj1, new_obj);
}

/* -------------------------
   | Combine two dialogues |
   ------------------------- */
int CombineDialogs(OBJECT **newtree, OBJECT *tree1, OBJECT *tree2)
{
int elem1, elem2, i;
GRECT fr;

DEBUG();
elem1 = 0;	while(!(tree1[elem1++].ob_flags & LASTOB));
elem2 = 0;	while(!(tree2[elem2++].ob_flags & LASTOB));

if (*newtree != tree1)
	{
	form_center(tree1, &fr);
	*newtree = (OBJECT *)malloc(sizeof(OBJECT) * (elem1 + elem2));
	memcpy(*newtree, tree1, sizeof(OBJECT) * elem1);
	}
else
	*newtree = (OBJECT *)realloc(tree1, sizeof(OBJECT) * (elem1 + elem2));

form_center(tree2, &fr);
tree2[0].ob_x = tree2[0].ob_y = 0;

memcpy(*newtree + elem1, tree2, sizeof(OBJECT) * elem2);
(*newtree)[elem1 - 1].ob_flags &= ~LASTOB;
(*newtree)[elem1 - 1].ob_flags |= 0x8000;

i = elem1;
while(elem2--)
	{
	if ((((*newtree)[i].ob_type & 0xFF) == G_FTEXT || 
			 ((*newtree)[i].ob_type & 0xFF) == G_FBOXTEXT) &&
			 ((*newtree)[i].ob_flags & EDITABLE))
		{
		(*newtree)[i].ob_type |= 0x80;
		(*newtree)[i].ob_flags &= ~EDITABLE;
		}

	if ((*newtree)[i].ob_next != -1) (*newtree)[i].ob_next += elem1;
	if ((*newtree)[i].ob_head != -1) (*newtree)[i].ob_head += elem1;
	if ((*newtree)[i].ob_tail != -1) (*newtree)[i].ob_tail += elem1;
	i++;		
	}
	
return elem1;
}

/* -------------------
   | Change tab page |
   ------------------- */
void SetTabPage(OBJECT *tree, int tabobj, int page, int obj, int num, int *pages)
{
int i;

DEBUG();
BindTab(tree, tabobj, page);
redraw_object(tree, tabobj);

for (i = 0; i < num; i++)
	{
	SetSelect(tree, pages[i], obj == pages[i]);
	redraw_object(tree, pages[i]);
	}
redraw_object(tree, obj);
}

