/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
 #include "extern.h"

extern int obj_vdi, obj_colours;

/* -------------------------
   | Compare files by name |
   ------------------------- */
int dir_namecmp(struct ftp_lsblk *a_file, struct ftp_lsblk *b_file)
{
DEBUG();
if (a_file->dir < b_file->dir)	return 1;
if (a_file->dir > b_file->dir)	return -1;

return strcmp(a_file->name, b_file->name);
}

/* -------------------------
   | Compare files by name |
   ------------------------- */
int dir_typecmp(struct ftp_lsblk *a_file, struct ftp_lsblk *b_file)
{
char *ad, *bd;
int cmp;

DEBUG();
if (a_file->dir < b_file->dir)	return 1;
if (a_file->dir > b_file->dir)	return -1;

ad = strchr(a_file->name + 1, '.');	if (!ad) ad = "\0";
bd = strchr(b_file->name + 1, '.');	if (!bd) bd = "\0";

cmp = strcmp(ad, bd);
if (cmp) return cmp;

return strcmp(a_file->name, b_file->name);
}

/* -------------------------
   | Compare files by name |
   ------------------------- */
int dir_datecmp(struct ftp_lsblk *a_file, struct ftp_lsblk *b_file)
{
DEBUG();
if (a_file->dir 	< b_file->dir)		return  1;
if (a_file->dir 	> b_file->dir)		return -1;
if (a_file->year 	< b_file->year)		return -1;
if (a_file->year 	> b_file->year)		return  1;
if (a_file->month < b_file->month)	return -1;
if (a_file->month > b_file->month)	return  1;
if (a_file->day 	< b_file->day)		return -1;
if (a_file->day 	> b_file->day)		return  1;
if (a_file->hour 	< b_file->hour)		return -1;
if (a_file->hour 	> b_file->hour)		return  1;
if (a_file->min   < b_file->min)		return -1;
if (a_file->min 	> b_file->min)		return  1;
if (a_file->sec 	< b_file->sec)		return -1;
if (a_file->sec 	> b_file->sec)		return  1;

return strcmp(a_file->name, b_file->name);
}

/* -------------------------
   | Compare files by name |
   ------------------------- */
int dir_sizecmp(struct ftp_lsblk *a_file, struct ftp_lsblk *b_file)
{
DEBUG();
if (a_file->dir < b_file->dir)		return 1;
if (a_file->dir > b_file->dir)		return -1;
if (a_file->size < b_file->size)	return -1;
if (a_file->size > b_file->size)	return 1;

return strcmp(a_file->name, b_file->name);
}

/* ------------------
   | Sort directory |
   ------------------ */
void sort_dir(int filecnt, struct ftp_lsblk *files, int sort)
{
int offset;

DEBUG();
qsort(files, filecnt, sizeof(struct ftp_lsblk), dir_namecmp);

offset = 0;
if (filecnt && !strcmp(files[0].name, ".")) offset++;
if (offset < filecnt && !strcmp(files[offset].name, "..")) offset++;

if (offset < filecnt)
	switch (sort)
		{
		case 0:								/* Sort name */ /* do nothing */
			break;
	
		case 1:								/* Sort extension */
			qsort(files + offset, filecnt - offset, sizeof(struct ftp_lsblk), dir_typecmp);
			break;
	
		case 2:								/* Sort date */
			qsort(files + offset, filecnt - offset, sizeof(struct ftp_lsblk), dir_datecmp);
			break;
	
		case 3:								/* Sort size */
			qsort(files + offset, filecnt - offset, sizeof(struct ftp_lsblk), dir_sizecmp);
			break;
		}
}

/* ------------------------
   | Paint directory view |
   ------------------------ */
int cdecl dir_view(PARMBLK *paramblock)
{
int xy[4], i, icn, cxy[4];
int filecnt, top, left;
struct ftp_lsblk *files;
GRECT r1, r2;
int bits, *sizes;
char buf[64];

DEBUG();
if (!paramblock->pb_parm)
	{
	filecnt = local_filecnt;
	files = localfiles;
	left = local_left * tx_width;
	top = local_filetop;
	bits = ftp_opts.local_bits;
	sizes = ftp_opts.local_sizes;
	}
else
	{
	filecnt = ftp_filecnt;
	files = ftpfiles;
	left = ftp_left * tx_width;
	top = ftp_filetop;
	bits = ftp_opts.ftp_bits;
	sizes = ftp_opts.ftp_sizes;
	}

r1.g_x = paramblock->pb_xc;
r1.g_y = paramblock->pb_yc;
r1.g_w = paramblock->pb_wc;
r1.g_h = paramblock->pb_hc;
r2.g_x = paramblock->pb_x;
r2.g_y = paramblock->pb_y;
r2.g_w = paramblock->pb_w;
r2.g_h = paramblock->pb_h;

if (!rc_intersect(&r1, &r2))
	return 0;
	
cxy[0] = r2.g_x;
cxy[1] = r2.g_y;
cxy[2] = r2.g_x + r2.g_w - 1;
cxy[3] = r2.g_y + r2.g_h - 1;

vs_clip(vdi_handle, TRUE, cxy);

xy[0] = paramblock->pb_x;
xy[1] = paramblock->pb_y;
xy[2] = xy[0] + paramblock->pb_w - 1;
xy[3] = xy[1] + paramblock->pb_h - 1;

vsf_interior(vdi_handle, FIS_HOLLOW);
vr_recfl(vdi_handle, xy);

vswr_mode(vdi_handle, MD_TRANS);

vst_color(vdi_handle, 1);
for (i = 0; i < (paramblock->pb_h + tx_height - 1) / tx_height && i + top < filecnt; i++)
	{
	int x;
	if (files[top + i].dir)
		{
		if (files[top + i].name[1] == '.' && files[top + i].name[0] == '.' && files[top + i].name[2] == 0)
			icn = 4;
		else
			icn = 0;
		}
	else
		if (files[top + i].link)
			icn = 3;
		else
			icn = 1;
			
	icon_copy(symbols, 0, (smallfont ? 8 : 16) * icn, 
					 paramblock->pb_x - left, paramblock->pb_y + tx_height * i, 
					 16, smallfont ? 8 : 16);
	x = paramblock->pb_x + 2 * 8 - left;
	
	strncpy(buf, files[top + i].name, sizes[0] + 1);
	if (buf[sizes[0]])
		{
		buf[sizes[0] - 1] = '~';
		buf[sizes[0]] = 0;
		}
	
	vst_color(vdi_handle, files[top + i].color);
		
	v_gtext(vdi_handle, x, paramblock->pb_y + tx_height * i, buf);
	x += tx_width * sizes[0] + tx_width;
	
	if (bits & DIR_SIZE)
		{
		if (!files[top + i].dir)
			sprintf(buf, "%ld", files[top + i].size);
		else
			strcpy(buf, "<dir>");
			
		if (sizes[1] > strlen(buf))
			v_gtext(vdi_handle, x + tx_width * (sizes[1] - strlen(buf)), paramblock->pb_y + tx_height * i, buf);
		else
			v_gtext(vdi_handle, x, paramblock->pb_y + tx_height * i, buf + strlen(buf) - sizes[1]);

		x += tx_width * sizes[1] + tx_width;
		}
		
	if (bits & DIR_DATE)
		{	
		if (sizes[2] <= 6)	/* Day / Month */
			sprintf(buf, "%02d%02d", files[top + i].month, files[top + i].day);
		else if (sizes[2] < 8)	/* Day / Month / Year */
			sprintf(buf, "%02d%02d%02d", files[top + i].year % 100, files[top + i].month, files[top + i].day);
		else if (sizes[2] < 10)	/* Day / Month / Year */
			sprintf(buf, "%04d%02d%02d", files[top + i].year, files[top + i].month, files[top + i].day);
		else if (sizes[2] < 12)	/* Year / Day / Month / Hour / Min */
			sprintf(buf, "%02d%02d%02d%02d%02d", files[top + i].year % 100, files[top + i].month, files[top + i].day, 
																					 files[top + i].hour, files[top + i].min);
		else if (sizes[2] < 16)	/* Day / Month / Year */
			sprintf(buf, "%04d%02d%02d%02d%02d", files[top + i].year, files[top + i].month, files[top + i].day, 
																					 files[top + i].hour, files[top + i].min);
		else 											/* Day / Month / Year */
			sprintf(buf, "%04d/%02d/%02d %02d:%02d", files[top + i].year, files[top + i].month, files[top + i].day, 
																					 files[top + i].hour, files[top + i].min);

		v_gtext(vdi_handle, x, paramblock->pb_y + tx_height * i, buf);
		x += tx_width * sizes[2] + tx_width;
		}
	
	if (bits & DIR_ATTR)
		{
		char attr[11];
		int i2;
		static char attrchr[] = "drwxrwxrwx";
		static int	attrbit[] = {S_IFDIR, 
														 S_IRUSR, S_IWUSR, S_IXUSR, 
														 S_IRGRP, S_IWGRP, S_IXGRP,
														 S_IROTH, S_IWOTH, S_IXOTH};

		
		attr[10] = 0;
	
		for (i2 = 0; i2 < 10; i2++)
			if (files[top + i].attr & attrbit[i2])
				attr[i2] = attrchr[i2];
			else
				attr[i2] = '-';

		v_gtext(vdi_handle, x, paramblock->pb_y + tx_height * i, attr);
		x += tx_width * sizes[3] + tx_width;
		}

	if (bits & DIR_USER)
		{
		v_gtext(vdi_handle, x, paramblock->pb_y + tx_height * i, files[top + i].user);
		x += tx_width * sizes[4] + tx_width;
		}

	if (bits & DIR_GROUP)
		{
		v_gtext(vdi_handle, x, paramblock->pb_y + tx_height * i, files[top + i].group);
		x += tx_width * sizes[5] + tx_width;
		}
		
	if (files[top + i].sel)
		{
		vswr_mode(vdi_handle, MD_XOR);
		
		xy[0] = paramblock->pb_x;
		xy[1] = paramblock->pb_y + tx_height * i;
		xy[2] = xy[0] + paramblock->pb_w - 3;
		xy[3] = xy[1] + tx_height - 1;
		vsf_interior(vdi_handle, FIS_SOLID);
		vsf_color(vdi_handle, 1);
		vr_recfl(vdi_handle, xy);
		
		vswr_mode(vdi_handle, MD_TRANS);
		}
	}
vswr_mode(vdi_handle, MD_REPLACE);
vst_color(vdi_handle, 1);

vs_clip(vdi_handle, FALSE, cxy);

return 0;
}

/* ------------------------
   | Set slider positions |
   ------------------------ */
void set_dirslider(int left, int redraw)
{
DEBUG();
if (left)
	set_slider_objects(maindial, LSLD, LSLDB, 
										 local_filecnt, local_filetop,
										 maindial[LDIR].ob_height / tx_height, redraw);
else
	set_slider_objects(maindial, RSLD, RSLDB, 
										 ftp_filecnt, ftp_filetop,
										 maindial[RDIR].ob_height / tx_height, redraw);
}

/* ------------------------------
   | Calculate horizontal width |
   ------------------------------ */
int calc_hwidth(int bits, int *sizes)
{
int i, w;

DEBUG();
for (i = 0, w = sizes[0] + 2; i < 6; i++)
	if (bits & (1 << i))
		{
		w++;
		w += sizes[i + 1];
		}
		
return w + 1;
}

/* ----------------------------------
   | Set horizontal slider position |
	 ---------------------------------- */
void set_dirhslider(int left, int redraw)
{
DEBUG();
if (left)
	{
	int w = calc_hwidth(ftp_opts.local_bits, ftp_opts.local_sizes);
	set_slider_objects(maindial, LSLDH, LSLDBH, 
										 w, local_left,
										 maindial[LDIR].ob_width / tx_width, redraw);
	}
else
	{
	int w = calc_hwidth(ftp_opts.ftp_bits, ftp_opts.ftp_sizes);
	set_slider_objects(maindial, RSLDH, RSLDBH, 
										 w, ftp_left,
										 maindial[RDIR].ob_width / tx_width, redraw);
	}
}

/* ------------------
   | Set sort order |
   ------------------ */
int select_sort(int sort)
{
static int t_sort[] = {SORT_NAME, SORT_EXT, SORT_DATE, SORT_SIZE};
static int sort_help[] = {SORT_NAME, SORT_EXT, SORT_DATE, SORT_SIZE,
													SORT_OKAY, SORT_CANCEL, 0};
OBJECT *tree;
int obj, i;

DEBUG();
rsrc_gaddr(R_TREE, SORTBY, &tree);

for (i = 0; i < 4; i++)
	SetSelect(tree, t_sort[i], i == sort);

open_kinddialog(tree, tree[SORT_TITLE].ob_spec.free_string, iconified, SMALLER|NAME|MOVER|CLOSER);
dlgset_help_info(tree, helpfile, sort_help, "SORT_");
obj = do_dialog(tree, 0, app_callback);
close_dialog(tree, TRUE);

if (obj == SORT_OKAY)
	{
	for (i = 0; i < 4; i++)
		if (IsSelected(tree, t_sort[i]))
			{
			sort = i;
			break;
			}

	return sort;
	}
else
	return -1;
}   

