/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
 #include "extern.h"

/* ----------------------------------
   | Analyse colors for local files |
   ---------------------------------- */
int colorRemoteFiles(void)
{
DEBUG();
return colorFiles(ftp_filecnt, ftpfiles,
							 	  local_filecnt, localfiles);
}

/* ----------------------
   | Read FTP directory |
   ---------------------- */
void ReadDirectory(void)
{
DEBUG();
SetInfo(readdir);
ftp_filetop = 0;

if (!ftp_ls(ftp_socket, &ftp_filecnt, &ftpfiles))
	form_alert(1, get_error);
else
	{
	int i;
	
	for (i = 0; i < ftp_filecnt; i++)
		if (!strcmp(ftpfiles[i].name, ".."))
			break;
	
	if (i == ftp_filecnt && ftp_remote[1])
		{
		if (!ftp_filecnt)
			ftpfiles = (struct ftp_lsblk *)malloc(sizeof(struct ftp_lsblk));
		else
			ftpfiles = (struct ftp_lsblk *)realloc(ftpfiles,
																						 sizeof(struct ftp_lsblk) *
																						 (ftp_filecnt + 1));
		ftpfiles[ftp_filecnt].dir = 1;
		ftpfiles[ftp_filecnt].link =
		ftpfiles[ftp_filecnt].size = 
		ftpfiles[ftp_filecnt].sel =
		ftpfiles[ftp_filecnt].hour =
		ftpfiles[ftp_filecnt].min =
		ftpfiles[ftp_filecnt].sec =
		ftpfiles[ftp_filecnt].day =
		ftpfiles[ftp_filecnt].month =
		ftpfiles[ftp_filecnt].year = 0;
		strcpy(ftpfiles[ftp_filecnt++].name, "..");
		}

	if (!act_conndef.show_dots)
		{
		i = 0;
		while(i < ftp_filecnt)
			{
			if (ftpfiles[i].name[0] == '.' && isalnum(ftpfiles[i].name[1]))
				{
				memmove(ftpfiles + i, ftpfiles + ftp_filecnt - 1, sizeof(struct ftp_lsblk));
				ftp_filecnt--;
				}
			else
				i++;
			}
		}
			
	if (!ftp_opts.v_links)
		{
		i = 0;
		while(i < ftp_filecnt)
			{
			if (ftpfiles[i].link)
				{
				memmove(ftpfiles + i, ftpfiles + ftp_filecnt - 1, sizeof(struct ftp_lsblk));
				ftp_filecnt--;
				}
			else
				i++;
			}
		}
					
	sort_dir(ftp_filecnt, ftpfiles, ftp_opts.r_sort);
	}
	
if (colorLocalFiles())
	redraw_object(maindial, LDIR);
	
colorRemoteFiles();
redraw_object(maindial, RDIR);
set_dirslider(FALSE, TRUE);

set_buttonstate();
}

/* -----------------------
   | Rename remote files |
   ----------------------- */
void RenameRemote(void)
{
int i, modified;

DEBUG();
modified = FALSE;
for (i = 0; i < ftp_filecnt; i++)
	if (ftpfiles[i].sel)
		if (GetRenamedName(ftpfiles[i].name))
			{
			if (ftp_rename(ftp_socket, ftpfiles[i].name, name_buffer))
				{
				strcpy(ftpfiles[i].name, name_buffer);
				modified = TRUE;
				ftpfiles[i].sel = 0;
				}
			}
			
if (modified)
	{
	sort_dir(ftp_filecnt, ftpfiles, ftp_opts.r_sort);
	if (colorLocalFiles())
		redraw_object(maindial, LDIR);
	redraw_object(maindial, RDIR);
	set_dirslider(FALSE, TRUE);
	}
}

/* -----------------------
   | Remove remote files |
   ----------------------- */
void RemoveRemote(int type)
{
int i, modified, obj, result;
OBJECT *tree;

DEBUG();
for (i = 0; i < ftp_filecnt; i++)
	if (ftpfiles[i].sel && ftpfiles[i].dir == type)
		break;
		
if (i == ftp_filecnt) return;

if (ftp_opts.v_delete)
	{
	static int del_help[] = {SUREYES, NOTSURE, 0};
	
	rsrc_gaddr(R_TREE, DELSURE, &tree);
	open_kinddialog(tree, tree[DEL_TITLE].ob_spec.free_string, iconified, SMALLER|NAME|MOVER|CLOSER);
	dlgset_help_info(tree, helpfile, del_help, "DEL_");
	obj = do_dialog(tree, 0, app_callback);
	close_dialog(tree, TRUE);
	
	if (obj != SUREYES) return;
	}
	
modified = FALSE;
for (i = 0; i < ftp_filecnt; i++)
	if (ftpfiles[i].sel && ftpfiles[i].dir == type)
		{
		if (type)
			result = ftp_rmdir(ftp_socket, ftpfiles[i].name);
		else
			result = ftp_unlink(ftp_socket, ftpfiles[i].name);
		
		if (result)
			{
			memcpy(ftpfiles + i, ftpfiles + ftp_filecnt - 1, 
						 sizeof(struct ftp_lsblk));
			ftp_filecnt--;
			modified = TRUE;
			ftp_filetop = 0;
			}
		}
			
if (modified)
	{
	sort_dir(ftp_filecnt, ftpfiles, ftp_opts.r_sort);
	if (colorLocalFiles())
		redraw_object(maindial, LDIR);
	redraw_object(maindial, RDIR);
	set_dirslider(FALSE, TRUE);
	set_buttonstate();
	}
}

/* -------------------------
   | Local directory popup |
   ------------------------- */
void remote_popup(void)
{
OBJECT *pop;

DEBUG();
rsrc_gaddr(R_TREE, R_POPUP, &pop);

if (!ftp_socket)
	DisableObj(pop, R_CHDIR);
else
	EnableObj(pop, R_CHDIR);
	
if (maindial[RMD].ob_state & DISABLED)
	DisableObj(pop, R_MKDIR);
else
	EnableObj(pop, R_MKDIR);

if (maindial[RRD].ob_state & DISABLED)
	DisableObj(pop, R_RMDIR);
else
	EnableObj(pop, R_RMDIR);

if (maindial[RREF].ob_state & DISABLED)
	{
	DisableObj(pop, R_REFRESH);
	DisableObj(pop, R_SITE);
	DisableObj(pop, R_QUOTE);
	}
else
	{
	EnableObj(pop, R_REFRESH);
	EnableObj(pop, R_SITE);
	EnableObj(pop, R_QUOTE);
	}

switch(popup(pop))
	{
	case R_SORT:
		{
		int sort = select_sort(ftp_opts.r_sort);
		if (sort != -1 && ftp_opts.r_sort != sort)
			{
			ftp_opts.r_sort = sort;
			sort_dir(ftp_filecnt, ftpfiles, ftp_opts.r_sort);
			redraw_object(maindial, RDIR);
			}
		break;
		}

	case R_CHDIR:
		if (InputSomething(chdir_label, ftp_remote, "CD_"))
			{
			ReadDirectory();
			SetInfo(done);
			}
		break;
		
	case R_MKDIR:
		if (GetNewFolderName())
			if (ftp_mkdir(ftp_socket, name_buffer))
				{
				ReadDirectory();
				SetInfo(done);
				}
		break;
		
	case R_RMDIR:
		if (ftp_socket)
			RemoveRemote(1);
		break;

	case R_REFRESH:
		ReadDirectory();
		SetInfo(done);
		break;
		
	case R_SITE:
		if (InputSomething(site_label, ftp_opts.site_cmd, "SITE_"))
			if (ftp_site(ftp_socket, ftp_opts.site_cmd))
				{
				ReadDirectory();
				SetInfo(done);
				}
		break;

	case R_QUOTE:
		if (InputSomething(quote_label, ftp_opts.quote_cmd, "QUOT_"))
			if (ftp_site(ftp_socket, ftp_opts.quote_cmd))
				{
				ReadDirectory();
				SetInfo(done);
				}
		break;
	}
}
