/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"
#include "ext.h"

static int ret_code, fblk_cnt = 0, fblk_max;
static int data_socket = 0;
static char ftp_buf[256];
struct ftp_lsblk *fblks;
static char *mnames[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
												 "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

/* Local prototypes */
static int TestIncoming(int s);
static int TestException(int s);

static int create_ffblk(char *buf, struct ftp_lsblk *fblk);

/* Used by create_ffblk */
static int 	ftp_idx[] = 	{0, 0, 
													 1, 2, 3,
													 4, 5, 6,
													 7, 8, 9,
													 9, 9, 
													 3, 3,
													 6, 6};
static char ftp_char[] = 	"dlrwxrwxrwxtTsSsS";
static int  ftp_attr[] =	{S_IFDIR,	S_IFLNK, 
													 S_IRUSR, S_IWUSR, S_IXUSR,
													 S_IRGRP, S_IWGRP, S_IXGRP,
													 S_IROTH, S_IWOTH, S_IXOTH,
													 S_IXOTH, 0,
													 S_IXUSR, 0,
													 S_IXGRP, 0};

extern char *logmem;

/* ----------------------
   | Connect FTP server |
   ---------------------- */
int ftp_connect(char *server, int port)
{
struct sockaddr_in remote;
struct hostent *host;
struct servent *serv;
unsigned long addr;
int s;

DEBUG();
if (isdigit(server[0]))
  {
  addr = inet_addr(server);
  remote.sin_family = AF_INET;
  memcpy(&remote.sin_addr, &addr, sizeof(long));
  }
else
  {
  host = gethostbyname(server);
  if (!host)
    {
    form_alert(1, unknown_host);
    return 0;
    }
  remote.sin_family = host->h_addrtype;
  memcpy(&remote.sin_addr, host->h_addr, host->h_length);
  }

if (!port)
	{
	serv = getservbyname("ftp", NULL);
	if (!serv)
  	{
	  form_alert(1, unknown_service);
	  return 0;
  	}
  port = serv->s_port;
	}

open_log();
s = socket(AF_INET, SOCK_STREAM, 0);

remote.sin_port = port;

if (!connect(s, (struct sockaddr *)&remote, (int)sizeof(remote)))
  {
	ret_code = ftp_return(s, ftp_buf, NULL);
	if (ret_code >= 200 && ret_code <= 299)
	  return s;
  }

close_socket(s);
close_log();
return 0;
}

/* -----------------------
   | Login to ftp server |
   ----------------------- */
int ftp_login(int s, char *user, char *pass)
{
char buf[256];

DEBUG();
sprintf(buf, "USER %s\r\n", user);
log_send(s, buf, strlen(buf), 0);

ret_code = ftp_return(s, ftp_buf, NULL);
if (ret_code == 331)
	{
	sprintf(buf, "PASS %s\r\n", pass);
	send(s, buf, strlen(buf), 0);
	logging("PASS password (not dumped)\r\n", -1);
	ret_code = ftp_return(s, ftp_buf, NULL);
	}

return (ret_code >= 200 && ret_code <= 299);
}

/* ------------------------
   | Close FTP connection |
   ------------------------ */
int ftp_close(int s)
{
DEBUG();
log_send(s, "QUIT\r\n", 6, 0);
ret_code = ftp_return(s, ftp_buf, NULL);

close_socket(s);
close_log();
return (ret_code >= 200 && ret_code <= 299);
}

/* ---------------------
   | Abort FTP command |
   --------------------- */
int ftp_abort(int s)
{
DEBUG();
log_send(s, "ABOR\r\n", 6, 0);
if (data_socket)
	{
	close_socket(data_socket);
	data_socket = 0;
	}
ret_code = ftp_return(s, ftp_buf, NULL);
return (ret_code >= 200 && ret_code <= 299);
}

/* ------------------------------
   | Get current work directory |
   ------------------------------ */
int ftp_getcwd(int s, char *buf)
{
int i, j, l;
char c, *m;

DEBUG();
log_send(s, "PWD\r\n", 5, 0);

ret_code = ftp_return(s, ftp_buf, NULL);
if (ret_code >= 200 && ret_code <= 299)
	{
	i = 0;
	while(!isspace(ftp_buf[i]))	i++;
	while(ftp_buf[i] != '/') i++;
	
	j = 0;
	if (ftp_buf[i - 1] == '"' || ftp_buf[i - 1] == '\'')
		{
		c = ftp_buf[i - 1];
		while(ftp_buf[i] && ftp_buf[i] != c)
			buf[j++] = ftp_buf[i++];
		}
	else
		{
		while(ftp_buf[i] && !isspace(ftp_buf[i]))
			buf[j++] = ftp_buf[i++];
		}
	buf[j] = 0;

	m = buf;
	l = (int)strlen(m);
	if (l > 28)
		m += l - 28;
	strcpy(maindial[RDIRINFO].ob_spec.tedinfo->te_ptext, m);
	redraw_object(maindial, RDIRINFO);
		
	return 1;
	}
else
	return 0;
}

/* ------------------------------
   | Set current work directory |
   ------------------------------ */
int ftp_setcwd(int s, char *buf)
{
DEBUG();
if (!strcmp(buf, ".."))
	{
	log_send(s, "CDUP\r\n", 6, 0);
	}
else
	{
	sprintf(ftp_buf, "CWD %s\r\n", buf);
	log_send(s, ftp_buf, strlen(ftp_buf), 0);
	}
ret_code = ftp_return(s, ftp_buf, NULL);

return (ret_code >= 200 && ret_code <= 299);
}

/* ------------------------------
   | Create a directory via ftp |
   ------------------------------ */
int ftp_mkdir(int s, char *buf)
{
DEBUG();
sprintf(ftp_buf, "MKD %s\r\n", buf);
log_send(s, ftp_buf, strlen(ftp_buf), 0);

ret_code = ftp_return(s, ftp_buf, NULL);
return (ret_code >= 200 && ret_code <= 299);
}

/* ------------------------------
   | Create a directory via ftp |
   ------------------------------ */
int ftp_unlink(int s, char *buf)
{
DEBUG();
sprintf(ftp_buf, "DELE %s\r\n", buf);
log_send(s, ftp_buf, strlen(ftp_buf), 0);

ret_code = ftp_return(s, ftp_buf, NULL);
return (ret_code >= 200 && ret_code <= 299);
}

/* ------------------------------
   | Remove a directory via ftp |
   ------------------------------ */
int ftp_rmdir(int s, char *buf)
{
DEBUG();
sprintf(ftp_buf, "RMD %s\r\n", buf);
log_send(s, ftp_buf, strlen(ftp_buf), 0);

ret_code = ftp_return(s, ftp_buf, NULL);
return (ret_code >= 200 && ret_code <= 299);
}

/* ---------------------------
   | Rename file through ftp |
   --------------------------- */
int ftp_rename(int s, char *old, char *new)
{
DEBUG();
sprintf(ftp_buf, "RNFR %s\r\n", old);
log_send(s, ftp_buf, strlen(ftp_buf), 0);

ret_code = ftp_return(s, ftp_buf, NULL);
if (ret_code == 350)
	{
	sprintf(ftp_buf, "RNTO %s\r\n", new);
	log_send(s, ftp_buf, strlen(ftp_buf), 0);
	
	ret_code = ftp_return(s, ftp_buf, NULL);
	return (ret_code >= 200 && ret_code <= 299);
	}
else
	return 0;
}

/* ---------------------------------
   | Set binary mode to TRUE/FALSE |
   --------------------------------- */
int ftp_binary(int s, int binflg)
{
DEBUG();
if (binflg)
	log_send(s, "TYPE I\r\n", 8, 0);
else
	log_send(s, "TYPE A\r\n", 8, 0);
ret_code = ftp_return(s, ftp_buf, NULL);
return (ret_code >= 200 && ret_code <= 299);
}

/* ---------------------------
   | Send communication port |
   --------------------------- */
static send_port(int s, int nsocket)
{
struct sockaddr_in nsock;
int i, nsocksz = (int)sizeof(nsock);
unsigned char *sptr;

DEBUG();
if (act_conndef.passive_transfer)
	{
	log_send(s, "PASV\r\n", 6, 0);
	ret_code = ftp_return(s, ftp_buf, NULL);
	if (ret_code >= 200 && ret_code <= 299)
		{
		nsock.sin_family = AF_INET;
		
		sptr = (unsigned char *)ftp_buf + 4;
		while (!isdigit(*sptr) && *sptr)
			sptr++;
		
		if (!*sptr) return FALSE;

		nsock.sin_addr.s_addr = 0L;
		for (i = 0; i < 4; i++)
			{
			nsock.sin_addr.s_addr <<= 8L;
			nsock.sin_addr.s_addr |= atoi((char *)sptr);
			while (*sptr != ',' && *sptr)
				sptr++;
				
			if (!*sptr || !isdigit(sptr[1])) return FALSE;
			sptr++;
			}
					
		nsock.sin_port = 0;
		for (i = 0; i < 2; i++)
			{
			nsock.sin_port <<= 8;
			nsock.sin_port |= atoi((char *)sptr);
			if (i == 1) break;
			
			while (*sptr != ',' && *sptr)
				sptr++;
				
			if (!*sptr || !isdigit(sptr[1])) return FALSE;
			sptr++;
			}

		if (!connect(nsocket, (struct sockaddr *)&nsock, (int)sizeof(nsock)))
  		return TRUE;
		}

	return FALSE;
	}
else
	{
	nsock.sin_family = AF_INET; /* Chg: */
	nsock.sin_port = 0; /* Chg: */
	nsock.sin_addr.s_addr = gethostip(); /* Chg: */
	bind(nsocket, (struct sockaddr *)&nsock, sizeof(nsock)); /* Chg: */
	listen(nsocket, 1);

	getsockname(nsocket, (struct sockaddr *)&nsock, &nsocksz);
	sptr = (unsigned char *)&nsock.sin_addr;
	strcpy(ftp_buf, "PORT ");
	for (i = 0; i < 4; i++)
		{
		itoa((int)sptr[i], ftp_buf + strlen(ftp_buf), 10);
		strcat(ftp_buf, ",");
		}		 
	
	sptr = (unsigned char *)&nsock.sin_port;
	itoa((int)sptr[0], ftp_buf + strlen(ftp_buf), 10);
	strcat(ftp_buf, ",");
	itoa((int)sptr[1], ftp_buf + strlen(ftp_buf), 10);
	strcat(ftp_buf, "\r\n");
	
	log_send(s, ftp_buf, strlen(ftp_buf), 0);
	ret_code = ftp_return(s, ftp_buf, NULL);
	return (ret_code >= 200 && ret_code <= 299);
	}
}

/* --------------------------------
   | Get first entry of directory |
   -------------------------------- */
int ftp_ls(int s, int *cnt, struct ftp_lsblk **fblk)
{
struct sockaddr_in nsock;
int nsocksz = (int)sizeof(nsock);
int nsocket, xmit_socket, exep;
SOCK_BUF lsbuf;

DEBUG();
if (fblk_cnt)
	{
	free(fblks);
	fblk_cnt = fblk_max = 0;
	}

/* Create data socket for listening */
nsocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

/* Send PORT id */
if (send_port(s, nsocket))
	{
	/* Get directory */
	log_send(s, "LIST\r\n", 6, 0);
	ret_code = ftp_return(s, ftp_buf, NULL);
	if (ret_code == 150)
		{
		if (act_conndef.passive_transfer)
			xmit_socket = nsocket;
		else
			{
			nsocksz = (int)sizeof(nsock);
			xmit_socket = accept(nsocket, (struct sockaddr *)&nsock, &nsocksz);
			close_socket(nsocket);
			}
			
		fblks = (struct ftp_lsblk *)malloc(sizeof(struct ftp_lsblk) *
																			 1024);
		fblk_max = 1024;

		lsbuf.ptr = lsbuf.curr = 0;
		lsbuf.size = 8000;		
		lsbuf.mem = (char *)malloc(lsbuf.size);
		
		exep = 0;
		do
			{
			while(TestIncoming(xmit_socket) || lsbuf.ptr < lsbuf.curr)
				{
				ftp_return(xmit_socket, ftp_buf, &lsbuf);
				
				if (ftp_opts.log_data)
					logging(ftp_buf, -1);
					
				if (create_ffblk(ftp_buf, fblks + fblk_cnt))
					{
					fblk_cnt++;
					if (fblk_cnt == fblk_max)
						fblks = (struct ftp_lsblk *)realloc(fblks,
												sizeof(struct ftp_lsblk) * (fblk_max += 1024));
					}
				}
			if (TestException(xmit_socket))
				exep = 1;
				
			}while(!exep || lsbuf.ptr < lsbuf.curr || TestIncoming(xmit_socket));
					
		free(lsbuf.mem);
		close_socket(xmit_socket);
		
		*cnt = fblk_cnt;
		*fblk = fblks;		
	
		ret_code = ftp_return(s, ftp_buf, NULL);
		return (ret_code >= 200 && ret_code <= 299);
		}
	}
close_socket(nsocket);
return 0;
}

/* --------------
   | Trim right |
   -------------- */
static void set_end(char *buf)
{
int l;

DEBUG();
l = (int)strlen(buf);
while(l && isspace(buf[l - 1]))
	l--;
buf[l] = 0;
}

/* --------------------------------
   | Convert into ffblk structure |
   -------------------------------- */
static int create_ffblk(char *buf, struct ftp_lsblk *fblk)
{
char *s, *s1;
int i, l, y = 0, m = 0, d = 0, attr, c;
char user[9], group[9];
struct date currdate;

DEBUG();
getdate(&currdate);

if (isdigit(*((unsigned char *)buf)))
	return 0;
	
fblk->dir = fblk->link = l = 0;
while(isspace(buf[l]))
	l++;
if (buf[l] == 'd') fblk->dir = 1;
if (buf[l] == 'l') fblk->link = 1;

attr = 0;
for (i = 0; i < 17; i++)
	if (buf[l + ftp_idx[i]] == ftp_char[i])
		attr |= ftp_attr[i];

set_end(buf);

s = strrchr(buf, ' ');
s1 = strchr(buf, ' ');
if (s && s1 != s)
	{
	strcpy(fblk->name, s + 1);	
	*s = 0;

	set_end(buf);
	
	while (*s1 && *s1 == ' ')
		s1++;

	s1 = strchr(s1, ' ');	/* Skip number of links */	
	while (*s1 && *s1 == ' ')
		s1++;
	
	s = strchr(s1, ' ');
	if (s > s1 + 8) c = 8; else c = (int)(s - s1);
	strncpy(user, s1, c);
	user[8] = 0;

	s1 = s;
	while (*s1 && *s1 == ' ')
		s1++;
	s = strchr(s1, ' ');
	if (s > s1 + 8) c = 8; else c = (int)(s - s1);
	strncpy(group, s1, c);
	group[8] = 0;
	
	l = (int)strlen(buf);	
	if (!strcmp(buf + l - 2, "->"))
		{
		buf[l - 2] = 0;
		set_end(buf);

		s = strrchr(buf, ' ');
		if (s)
			{
			strcpy(fblk->name, s + 1);	
			*s = 0;
			set_end(buf);
			}
		}
		
	s = strrchr(buf, ' ');
	if (s)
		{
		*s++ = 0;
		set_end(buf);

		if (strchr(s, ':'))
			{
			fblk->hour = 0;
			while(isdigit(*s))
				fblk->hour = fblk->hour * 10 + *s++ - '0';
			s++;
			fblk->min = 0;
			while(isdigit(*s))
				fblk->min = fblk->min * 10 + *s++ - '0';
			
			fblk->sec = 0;
			if (*s == ':')
				{
				s++;
				while(isdigit(*s))
					fblk->sec = fblk->sec * 10 + *s++ - '0';
				}
			
			y = currdate.da_year;
			}
		else
			{
			fblk->hour = fblk->min = fblk->sec = 0;
			y = atoi(s);
			}
			
		s = strrchr(buf, ' ');
		if (s)
			{
			*s++ = 0;
			set_end(buf);
			
			d = atoi(s);
			
			s = strrchr(buf, ' ');
			if (s)
				{
				*s++ = 0;
				set_end(buf);
				
				for (m = 0; m < 12; m++)
					if (!stricmp(s, mnames[m]))
						{
						m++;
						break;
						}
				if (m > currdate.da_mon)
					y--;

				
				s = strrchr(buf, ' ');
				if (s)
					fblk->size = atol(s + 1);
				else
					fblk->size = 0;
				}
			}
		}
	
	strcpy(fblk->user, user);
	strcpy(fblk->group, group);
	fblk->sel = 0;					
	fblk->year = y;
	fblk->month = m;
	fblk->day = d;
	fblk->attr = attr;
	
	return 1;
	}
else
	return 0;
}

/* -------------------
   | Incoming data ? |
   ------------------- */
static int TestIncoming(int s)
{
int nx;
struct fd_set fd_chk;
struct timeval tval;

DEBUG();
FD_ZERO(&fd_chk);
FD_SET(s, &fd_chk);
		
tval.tv_sec = 0;
tval.tv_usec = 0;
		
nx = select(s + 1, &fd_chk, NULL, NULL, &tval);
return (nx && FD_ISSET(s, &fd_chk));
}

/* ---------------------
   | Socket exception? |
   --------------------- */
static int TestException(int s)
{
#ifdef MINT_GW
long prev;
#endif
int nx;
struct fd_set fd_exep;
struct timeval tval;

DEBUG();
FD_ZERO(&fd_exep);
FD_SET(s, &fd_exep);
		
tval.tv_sec = 0;
tval.tv_usec = 0;
#ifdef MINT_GW
    prev = sockfcntl(s, SO_GETFL, SO_O_NONBLOCK);
    sockfcntl(s, SO_SETFL, SO_O_NONBLOCK);
#endif		
nx = select(s + 1, NULL, NULL, &fd_exep, &tval);
#ifdef MINT_GW
    sockfcntl(s, SO_SETFL, prev);
#endif
return (nx && FD_ISSET(s, &fd_exep));
}

/* ------------------------
   | Restart transmission |
   ------------------------ */
int ftp_restart(int s, long pos)
{
DEBUG();
sprintf(ftp_buf, "REST %ld\r\n", pos);
log_send(s, ftp_buf, strlen(ftp_buf), 0);

ret_code = ftp_return(s, ftp_buf, NULL);
return (ret_code == 350);
}

/* -----------------
   | Quote command |   
   ----------------- */
int ftp_quote(int s, char *quote)
{
DEBUG();
if (!logmem) 
	{
	logmem = (char *)malloc(1);
	if (logmem) *logmem = 0;
	}
	
sprintf(ftp_buf, "QUOTE %s\r\n", quote);
log_send(s, ftp_buf, strlen(ftp_buf), 0);

ret_code = ftp_return(s, ftp_buf, NULL);

view_file(NULL, FALSE, quote_label);

return (ret_code >= 200 && ret_code <= 299);
}
   
/* ----------------
   | Site command |   
   ---------------- */
int ftp_site(int s, char *site)
{
DEBUG();
if (!logmem) 
	{
	logmem = (char *)malloc(1);
	if (logmem) *logmem = 0;
	}

sprintf(ftp_buf, "SITE %s\r\n", site);
log_send(s, ftp_buf, strlen(ftp_buf), 0);

ret_code = ftp_return(s, ftp_buf, NULL);

view_file(NULL, FALSE, site_label);

return (ret_code >= 200 && ret_code <= 299);
}

/* ---------------------------
   | Open receive connection |
   --------------------------- */
int ftp_retrieve(int s, char *remotefile, long offset)
{
struct sockaddr_in nsock;
int nsocksz = (int)sizeof(nsock);
int nsocket;

DEBUG();
/* Create data socket for listening */
nsocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

/* Send PORT id */
if (send_port(s, nsocket))
	{
	if (offset)
		{
		if (!ftp_restart(s, offset))
			{
			close_socket(nsocket);
			return -2;
			}
		}
		
	sprintf(ftp_buf, "RETR %s\r\n", remotefile);
	log_send(s, ftp_buf, strlen(ftp_buf), 0);
	ret_code = ftp_return(s, ftp_buf, NULL);
	if (ret_code == 150)
		{
		if (act_conndef.passive_transfer)
			data_socket = nsocket;
		else
			{
			nsocksz = (int)sizeof(nsock);
			data_socket = accept(nsocket, (struct sockaddr *)&nsock, &nsocksz);
			close_socket(nsocket);
			}
		return 1;
		}
	}
close_socket(nsocket);
return 0;
}

/* ----------------------------
   | Receive block connection |
   ---------------------------- */
int ftp_getblock(int s, int bufsize, void *buffer)
{
int l, in, exep;

DEBUG();
in = 0;
if (data_socket)
	{
	exep = 0;
	do
		{
		while(TestIncoming(data_socket))
			{
			l = (int)recv(data_socket, NULL, bufsize - in, MSG_PEEK);
			if (l > 0)
				{
				l = (int)recv(data_socket, ((char *)buffer) + in, l, 0);
				in += l;
				if (in == bufsize)
					break;
				}
			}
		
		if (in < bufsize)	
			if (!TestIncoming(data_socket))
				if (TestException(data_socket))
					exep = 1;
			
		}while(in < bufsize && !exep);
	
	if (ftp_opts.log_data)
		log_dump(buffer, in);
	
	if (exep)
		{
		close_socket(data_socket);
		data_socket = 0;

		ret_code = ftp_return(s, ftp_buf, NULL);
		}
	}
else
	{
	if (ret_code >= 200 && ret_code <= 299)
		return 0;
	else
		return -1;
	}
	
return in;
}

/* --------------------------
   | Open output connection |
   -------------------------- */
int ftp_store(int s, char *remotefile, long offset)
{
struct sockaddr_in nsock;
int nsocksz = (int)sizeof(nsock);
int nsocket;

DEBUG();
/* Create data socket for listening */
nsocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

/* Send PORT id */
if (send_port(s, nsocket))
	{
	if (offset)
		{
		if (!ftp_restart(s, offset))
			{
			close_socket(nsocket);
			return -2;
			}
		}

	sprintf(ftp_buf, "STOR %s\r\n", remotefile);
	log_send(s, ftp_buf, strlen(ftp_buf), 0);
	ret_code = ftp_return(s, ftp_buf, NULL);
	if (ret_code == 150)
		{
		if (act_conndef.passive_transfer)
			data_socket = nsocket;
		else
			{
			nsocksz = (int)sizeof(nsock);
			data_socket = accept(nsocket, (struct sockaddr *)&nsock, &nsocksz);
			close_socket(nsocket);
			}
			
		return 1;
		}
	}
close_socket(nsocket);
return 0;
}

/* ----------------------------
   | Send block to connection |
   ---------------------------- */
int ftp_putblock(int s, int bufsize, void *buffer)
{
int l;

DEBUG();
if (!bufsize)
	{
	close_socket(data_socket);

	data_socket = 0;
	ret_code = ftp_return(s, ftp_buf, NULL);
	}

if (data_socket)
	{
	if (ftp_opts.log_data)
		log_dump(buffer, bufsize);

	l = (int)send(data_socket, buffer, bufsize, 0);
	if (TestException(data_socket))
		{
		close_socket(data_socket);
		data_socket = 0;

		ret_code = ftp_return(s, ftp_buf, NULL);
		}
	}
else
	{
	if (ret_code >= 200 && ret_code <= 299)
		return 0;
	else
		return -1;
	}

return l;
}

/* -------------------------
   | Get FTP return string |
   ------------------------- */
int ftp_return(int s, char *buf, SOCK_BUF *sbuf)
{
int n;
int l, exep = 0;
char c;

DEBUG();
do
	{
	c = 0;
	n = 0;
	do
		{
		if (sbuf)
			{
			if (sbuf->ptr < sbuf->curr)
				{
				buf[n++] = c = sbuf->mem[sbuf->ptr++];
				}
			else
				{
				if (TestIncoming(s))
					{
					sbuf->curr = (int)recv(s, sbuf->mem, sbuf->size, 0);
					sbuf->ptr = 0;
					}
				else
					{
					if (TestException(s))
						exep = 1;
					}
				}
			}
		else
			{
			if (TestIncoming(s))
				{
				l = (int)recv(s, &c, 1, 0);
				if (l > 0)
					buf[n++] = c;
				else
					c = 0;
				}
			else
				{
				if (TestException(s))
					exep = 1;
				}
			}
		
		if (exep && !TestIncoming)
			break;
			
		}while(c != '\n');
		
	buf[n++] = 0;

	if (logmem)
		logging(buf, -1);
	else
		switch(buf[0])
			{
			case '1':
			case '2':
			case '3':
				if (ftp_opts.log_cmds)
					logging(buf, -1);
				break;
			
			case '4':
			case '5':
				if (ftp_opts.log_error)
					logging(buf, -1);
				break;
			}

	}while(isdigit(buf[0]) && buf[3] == '-');

return atoi(buf);
}

