/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

USERBLK grafbutts[8];
int buttset[] = {LDEL, 	RDEL, LREN, RREN, LVIEW, 	RVIEW,
								 LMD, 	RMD, 	LRD, 	RRD, 	LREF, 	RREF, LINFO,
								 RINFO};
int buttidx[] = {2,    	2,   	1,    1,    0,     	0,
								 3,			3,	 	4,   	4,    5,    	6,    7,
								 7};
int buttpos[] = {   2,     2,     1,   1,     0,      0,
                    3,     3,     4,   4,    5, 			5, 		6, 
                    6};

extern int min_dialwidth;
extern int min_dialheight;

/* -------------------------------
   | Initialize grafical buttons |
	 ------------------------------- */
void init_grfbuttons(void)
{
int i;

DEBUG();
for (i = 0; i < 14; i++)
	{
	maindial[buttset[i]].ob_type = G_USERDEF;
	maindial[buttset[i]].ob_spec.userblk = &grafbutts[buttidx[i]];
	}
for (i = 0; i < 8; i++)
	grafbutts[i].ub_code = graf_butt;

position_grfbuttons();
}

/* -----------------------------
   | Position grafical buttons |
	 ----------------------------- */
void position_grfbuttons(void)
{
int i;

DEBUG();
for (i = 0; i < 8; i++)
	{
	grafbutts[i].ub_parm = i * but_palheader.height * 3;
	}
for (i = 0; i < 14; i++)
	{
	maindial[buttset[i]].ob_x = buttpos[i] * but_palheader.width;
	maindial[buttset[i]].ob_width = but_palheader.width;
	maindial[buttset[i]].ob_height = but_palheader.height;
	}

maindial[LACTIONS].ob_height = but_palheader.height;
maindial[LACTIONS].ob_width  = but_palheader.width * 7;
maindial[RACTIONS].ob_height = but_palheader.height;
maindial[RACTIONS].ob_width  = but_palheader.width * 7;
}

/* --------------------
   | Graphical button |
   -------------------- */
int cdecl graf_butt(PARMBLK *paramblock)
{
int offset;

DEBUG();
obj_clip(paramblock, TRUE);

offset = paramblock->pb_parm;

if (paramblock->pb_currstate & DISABLED)
	offset += but_palheader.height * 2;
else
	if (paramblock->pb_currstate & SELECTED)
		offset += but_palheader.height;

icon_copy(buttons, 0, offset,
				  paramblock->pb_x, paramblock->pb_y, 
				  but_palheader.width, but_palheader.height);

obj_clip(paramblock, FALSE);

return 0;
}

static void enable(OBJECT *tree, int idx)
{
int s = tree[idx].ob_state;

if (s & DISABLED)
	change_object(tree, idx, s & ~DISABLED);
}

static void disable(OBJECT *tree, int idx)
{
int s = tree[idx].ob_state;

if (!(s & DISABLED))
	change_object(tree, idx, s | DISABLED);
}

/* ------------------------
   | Set the button state |
   ------------------------ */
void set_buttonstate(void)
{
int filenum, dirnum, i;

DEBUG();
dirnum = filenum = 0;
for (i = 0; i < local_filecnt; i++)
	if (localfiles[i].sel)
		{
		filenum++;
		if (localfiles[i].dir)
			dirnum++;
		else
			filenum++;
				
		if (filenum == 2 && dirnum) break;
		}

if (dirnum + filenum >= 1)
	{
	enable(maindial, LDEL);
	enable(maindial, LREN);
	enable(maindial, LVIEW);
	enable(maindial, LINFO);
	}
else
	{
	disable(maindial, LDEL);
	disable(maindial, LREN);
	disable(maindial, LVIEW);
	disable(maindial, LINFO);
	}

if (dirnum >= 1)
	{
	enable(maindial, LRD);
	}
else
	{
	disable(maindial, LRD);
	}
	
dirnum = filenum = 0;
if (ftp_socket)
	for (i = 0; i < ftp_filecnt; i++)
		if (ftpfiles[i].sel)
			{
			filenum++;
			if (ftpfiles[i].dir)
				dirnum++;
			else
				filenum++;
					
			if (filenum == 2 && dirnum) break;
			}

if (dirnum + filenum >= 1)
	{
	enable(maindial, RDEL);
	enable(maindial, RREN);
	enable(maindial, RVIEW);
	enable(maindial, RINFO);
	}
else
	{
	disable(maindial, RDEL);
	disable(maindial, RREN);
	disable(maindial, RVIEW);
	disable(maindial, RINFO);
	}

if (dirnum >= 1)
	{
	enable(maindial, RRD);
	}
else
	{
	disable(maindial, RRD);
	}

if (ftp_socket)
	{
	enable(maindial, RREF);
	enable(maindial, RMD);
	}
else
	{
	disable(maindial, RREF);
	disable(maindial, RMD);
	}
}
