/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
 #include "extern.h"

#define DP_NAMEMAX 3	/* Dpathconf */
#define DP_TRUNC       5
#define DP_NOTRUNC    0
#define DP_AUTOTRUNC  1
#define DP_DOSTRUNC   2		

/* ------------------------------------
   | Check Filesystem/Path parameters |
   ------------------------------------ */
void CheckFileSystem(char *chk_file, int *maxnamelen, int *dos_mode)
{
long err;

DEBUG();
err = Dpathconf(chk_file, DP_NAMEMAX);
if	(err > 0L)
	{
	*maxnamelen = (int) err;
	*dos_mode = (Dpathconf(chk_file, DP_TRUNC) == DP_DOSTRUNC);
	}
else	
	{
	*dos_mode = TRUE;
	*maxnamelen = 12;
	}
}

/* -----------------------------
   | Local directory selection |
   ----------------------------- */
int local_dirsel(void)
{
char file[64], path[256], *s;
int button;

DEBUG();
strcpy(path, local_dir);
strcat(path, "\\*.*");

file[0] = 0;

if (aes_version >= 0x104)
	fsel_exinput(path, file, &button, "Select directory");
else
	fsel_input(path, file, &button);
	
if (button == 1)
	{
	strcpy(local_dir, path);
	s = strrchr(local_dir, '\\');
	if (s)
		*s = 0;
	else
		{
		Dgetpath(local_dir + 2, 0);
		local_dir[1] = ':';
		local_dir[0] = Dgetdrv() + 'A';
		}
				
	return TRUE;
	}
return FALSE;
}

/* ----------------------------------
   | Analyse colors for local files |
   ---------------------------------- */
int colorLocalFiles(void)
{
DEBUG();
return colorFiles(local_filecnt, localfiles,
							 	  ftp_filecnt, ftpfiles);
}

/* -----------------------
   | Get local directory |
   ----------------------- */
void GetLocaldir(void)
{
DTA mydta;
char *m, fb[256];
long dirhandle, err;
char *fnamebuf;
int maxnamelen, dosmode;
XATTR xattr;
int fblk_max, l;
struct ftp_lsblk *lfs;

DEBUG();
if (local_dir[strlen(local_dir) - 1] == '\\')
	*strrchr(local_dir, '\\') = 0;

m = local_dir;
l = (int)strlen(m);
if (l > 28)
	m += l - 28;

strcpy(maindial[LDIRINFO].ob_spec.tedinfo->te_ptext, m);

if (l == 2 && local_dir[1] == ':')
	strcat(maindial[LDIRINFO].ob_spec.tedinfo->te_ptext, "\\");

redraw_object(maindial, LDIRINFO);
	
CheckFileSystem(local_dir, &maxnamelen, &dosmode);
fnamebuf = (char *)malloc(maxnamelen + 5);

strcpy(fb, local_dir);
strcat(fb, "\\");
m = strrchr(fb, '\\');

if (local_filecnt)
	{
	local_filecnt = 0;
	free(localfiles);
	}
local_filetop = 0;

localfiles = (struct ftp_lsblk *)malloc(sizeof(struct ftp_lsblk) *
						 													  1024);
fblk_max = 1024;

if (fnamebuf)
	{
	if (dosmode)
		{
		Fsetdta(&mydta);
		dirhandle = 1;
		}
	else
		dirhandle = Dopendir(fb, 0);
	if	(dirhandle >= 0)
		{
		do	
			{
			if (dosmode)
				{
				if (dirhandle == 1)
					{
					strcpy(m + 1, "*.*");
					err = Fsfirst(fb, 0xFF);
					dirhandle = 2;
					}
				else
					err = Fsnext();
					
				if (!err) strcpy(fnamebuf + 4, mydta.d_fname);
				}
			else
				err = Dreaddir(maxnamelen + 5, dirhandle, fnamebuf);

			if	(!err)
				{
				if (dosmode && mydta.d_attrib & FA_LABEL) continue;
				
				if (local_filecnt == fblk_max)
					localfiles = (struct ftp_lsblk *)realloc(localfiles,
											sizeof(struct ftp_lsblk) * (fblk_max += 1024));


				lfs = localfiles + local_filecnt;
				if (dosmode)
					{
					lfs->dir 	= (mydta.d_attrib & FA_DIREC) ? 1 : 0;

					lfs->hour = (mydta.d_time & 0xF800) >> 11;
					lfs->min 	= (mydta.d_time & 0x7E0) >> 5;
					lfs->sec 	= (mydta.d_time & 0x1F) * 2;

					lfs->day	= (mydta.d_date & 0x1F);
					lfs->month= (mydta.d_date & 0x1E0) >> 5;
					lfs->year	= ((mydta.d_date & 0xFE00) >> 9) + 1980;
							
					lfs->size	= mydta.d_length;
					
					lfs->attr = 0;
					
					if (mydta.d_attrib & FA_DIREC)
						lfs->attr |= S_IFDIR;
					
					if (mydta.d_attrib & FA_RDONLY)
						lfs->attr |= S_IRUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH;
					else
						lfs->attr |= DEFAULT_DIRMODE;
					}

				lfs->link	= 0;
				lfs->sel = 0;
				strcpy(lfs->name, fnamebuf + 4);
				
				local_filecnt++;
				}
			}while(!err);
			
		if (!dosmode)
		Dclosedir(dirhandle);
		}	
	free(fnamebuf);
	}

if (!local_filecnt)
	free(localfiles);
	
if (!dosmode)
	{
	lfs = localfiles;
	while (lfs < localfiles + local_filecnt)
		{
		strcpy(m + 1, lfs->name);
		Fxattr(0, fb, &xattr);
	
		lfs->dir 	= (xattr.attr  & FA_DIREC) ? 1 : 0;

		lfs->hour = (xattr.mtime & 0xF800) >> 11;
		lfs->min 	= (xattr.mtime & 0x7E0) >> 5;
		lfs->sec 	= (xattr.mtime & 0x1F) * 2;

		lfs->day	= (xattr.mdate & 0x1F);
		lfs->month= (xattr.mdate & 0x1E0) >> 5;
		lfs->year	= ((xattr.mdate & 0xFE00) >> 9) + 1980;
				
		lfs->size 			= xattr.size;

		lfs->attr				= xattr.mode;
		
		lfs++;
		}
	}

sort_dir(local_filecnt, localfiles, ftp_opts.l_sort);
colorLocalFiles();
if (colorRemoteFiles())
	redraw_object(maindial, RDIR);
redraw_object(maindial, LDIR);
set_dirslider(TRUE, TRUE);

set_buttonstate();
}

/* --------------------------
   | Create local directory |
   -------------------------- */
int	MakeLocalDir(void)
{
char fb[256];

DEBUG();
strcpy(fb, local_dir);
strcat(fb, "\\");
strcat(fb, name_buffer);
return Dcreate(fb) == 0;
}

/* ----------------------
   | Rename local files |
   ---------------------- */
void RenameLocals(void)
{
int i, modified;
char fb1[256], fb2[256];

DEBUG();
modified = FALSE;
for (i = 0; i < local_filecnt; i++)
	if (localfiles[i].sel)
		if (GetRenamedName(localfiles[i].name))
			{
			strcpy(fb1, local_dir);
			strcat(fb1, "\\");
			strcat(fb1, localfiles[i].name);
			
			strcpy(fb2, local_dir);
			strcat(fb2, "\\");
			strcat(fb2, name_buffer);

			if (!rename(fb1, fb2))
				{
				strcpy(localfiles[i].name, name_buffer);
				modified = TRUE;
				localfiles[i].sel = 0;
				}
			}
			
if (modified)
	{
	sort_dir(local_filecnt, localfiles, ftp_opts.l_sort);
	if (colorRemoteFiles())
		redraw_object(maindial, RDIR);
	redraw_object(maindial, LDIR);
	set_dirslider(TRUE, TRUE);
	}
}

/* ----------------------
   | Remove local files |
   ---------------------- */
void RemoveLocals(int type)
{
char fb[256];
int i, modified, obj, result;
OBJECT *tree;

DEBUG();
for (i = 0; i < local_filecnt; i++)
	if (localfiles[i].sel && localfiles[i].dir == type)
		break;
		
if (i == local_filecnt) return;

if (ftp_opts.v_delete)
	{
	static int del_help[] = {SUREYES, NOTSURE, 0};
	
	rsrc_gaddr(R_TREE, DELSURE, &tree);
	open_kinddialog(tree, tree[DEL_TITLE].ob_spec.free_string, iconified, SMALLER|NAME|MOVER|CLOSER);
	dlgset_help_info(tree, helpfile, del_help, "DEL_");
	obj = do_dialog(tree, 0, app_callback);
	close_dialog(tree, TRUE);
	
	if (obj != SUREYES) return;
	}
	
modified = FALSE;
for (i = 0; i < local_filecnt; i++)
	if (localfiles[i].sel && localfiles[i].dir == type)
		{
		strcpy(fb, local_dir);
		strcat(fb, "\\");
		strcat(fb, localfiles[i].name);

		if (!type)
#ifndef __AHCC__
			result = unlink(fb);
#else
			result = Fdelete(fb);
#endif
		else
			result = Ddelete(fb);
			
		if (!result);
			{
			memcpy(localfiles + i, localfiles + local_filecnt - 1, 
						 sizeof(struct ftp_lsblk));
			local_filecnt--;
			modified = TRUE;
			local_filetop = 0;
			}
		}
			
if (modified)
	{
	sort_dir(local_filecnt, localfiles, ftp_opts.l_sort);
	if (colorRemoteFiles())
		redraw_object(maindial, RDIR);
	redraw_object(maindial, LDIR);
	set_dirslider(TRUE, TRUE);
	set_buttonstate();
	}
}

/* -------------------------
   | Local directory popup |
   ------------------------- */
void local_popup(void)
{
OBJECT *pop;

DEBUG();
rsrc_gaddr(R_TREE, L_POPUP, &pop);

if (maindial[LRD].ob_state & DISABLED)
	DisableObj(pop, L_RMDIR);
else
	EnableObj(pop, L_RMDIR);
	
switch(popup(pop))
	{
	case L_SORT:
		{
		int sort = select_sort(ftp_opts.l_sort);
		if (sort != -1 && ftp_opts.l_sort != sort)
			{
			ftp_opts.l_sort = sort;
			sort_dir(local_filecnt, localfiles, ftp_opts.l_sort);
			redraw_object(maindial, LDIR);
			}
		break;
		}

	case L_CHDIR:
		if (local_dirsel())
			GetLocaldir();
		break;
		
	case L_MKDIR:
		if (GetNewFolderName())
			{
			MakeLocalDir();
			GetLocaldir();
			}
		break;
		
	case L_RMDIR:
		RemoveLocals(1);
		break;
		
	case L_REFRESH:
		GetLocaldir();
		break;
	}
}
