/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static int log_handle = 0;
char *logmem = NULL;

/* ------------------------------
   | Open log file if activated |
   ------------------------------ */
void open_log(void)
{
long fh;

DEBUG();
if (!ftp_opts.enable_log) return;

if (!ftp_opts.logfile[0])
	{
	form_alert(1, no_logfile);
	return;
	}
	
fh = Fopen(ftp_opts.logfile, FO_RW);
if (fh == -33)
	fh = Fcreate(ftp_opts.logfile, 0);
	
if (fh > 0)
	{
	log_handle = (int)fh;
	Fseek(0L, log_handle, 2);
	}
else
	form_alert(1, cantopen_log);
}

/* --------------------------------
   | Close log file if it is open |
   -------------------------------- */
void close_log(void)
{
DEBUG();
if (log_handle)
	{
	Fclose(log_handle);
	log_handle = 0;
	}
}

/* -------------
   | Dump data |
   ------------- */
void log_dump(unsigned char *buffer, int size)
{
char buf[80];
unsigned char c;
int i, j;
static char hexchar[] = "0123456789ABCDEF";

DEBUG();
if (!ftp_opts.enable_log || !log_handle) return;

if (size > 0)
	{
	for (i = 0; i < size; i += 16)
		{
		memset(buf, ' ', 66);
		buf[66] = 0;
	
		for (j = 0; j < 16 && i + j < size; j++)
			{
			c = buffer[i + j];
			
			buf[j * 3] = hexchar[(c >> 4)];
			buf[j * 3 + 1] = hexchar[c & 0xF];
			
			if (!isprint(c)) c = '.';

			buf[50 + j] = c;
			}
		Fwrite(log_handle, strlen(buf), buf);
		Fwrite(log_handle, 2, "\r\n");
		}
	}
}
   
/* -------------------------
   | Log to specified file |
   ------------------------- */
void logging(char *buffer, int len)
{
DEBUG();
if (!logmem && (!ftp_opts.enable_log || !log_handle)) return;

if (len)
	{
	if (len < 0)
		len = (int)strlen(buffer);

	if (logmem)
		{
		logmem = (char *)realloc(logmem, strlen(logmem) + len + 1);
		strncat(logmem, buffer, len);
		}
	
	if (!ftp_opts.enable_log || !log_handle) return;

	Fwrite(log_handle, len, buffer);
	}
}

/* --------------------------------------------
   | Log all sended command if log is enabled |
   -------------------------------------------- */
long log_send(int sfd, void *buf, long len, unsigned flags)
{
DEBUG();
if (ftp_opts.log_cmds || logmem)
	logging(buf, -1);

return send(sfd, buf, len, flags);
}
