/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"
#include "global.h"
#include "vaproto.h"

#ifdef TRAP_LOG
#  include <setjmp.h>
#endif

#ifdef EVAL
#  define WAIT_ABOUT
#  define WAIT_ABOUT2
#	 define WAIT_FLG 1
#endif

USERBLK ldirv_ublk 	= {dir_view, 0L};
USERBLK rdirv_ublk 	= {dir_view, 1L};

int min_dialwidth;
int min_dialheight;

static int mainhelp_ids[] = {LDIRINFO, 
														 RDIRINFO,
														 SEND, RECEIVE,
														 FCONNECT, FDISCONN,
														 TYPEPANEL, ASCIMODE,
														 BINMODE, AUTOMODE,
														 RACTIONS, RVIEW,
														 RREN, RDEL, RMD,
														 RRD, RREF, RINFO,
														 LACTIONS, LVIEW,
														 LREN, LDEL, LMD,
														 LRD, LREF, LINFO,
														 0};
														 
/* -----------------------------
   | Get installed TOS version |
   ----------------------------- */
void get_aesversion(void)
{
long old_stk;

old_stk = Super(0L);

aes_version = (*(SYSHDR **)(0x4F2))->os_version;

Super((void *)old_stk);
}

/* ---------------------------- */
/* | Arbeitsstation schlieen | */
/* ---------------------------- */
void cls_vwork(void)
{
DEBUG();
v_clsvwk(vdi_handle);
}

/* ---------------------------- */
/* | Virtuelle Arbeitsstation | */
/* ---------------------------- */
void open_vwork(void)
{
int i;
int dummy;
int wout[57], t_attr[10];
static int colnum[] = {2, 4, 16, 256, 0};

DEBUG();
vdi_handle = graf_handle(&dummy, &dummy, &dummy, &dummy);

for(i = 0; i < 10; work_in[i++] = 1);
work_in[10] = 2;

v_opnvwk(work_in, &vdi_handle, work_out);

vq_extnd(vdi_handle, TRUE, wout);
planes = wout[4];
colours = work_out[13];
if (planes >= 16)
	dither_colours = 0;
else
	dither_colours = colours;

for (i = 0; i < 5; i++)
	if (dither_colours == colnum[i])
		dither_index = i;

vqt_attributes(vdi_handle, t_attr);
default_fsize = t_attr[7];
tx_width = t_attr[8];
tx_height = t_attr[9];

detect_pixeltype();

vst_alignment(vdi_handle, 0, 5, &dummy, &dummy);
}

static void cdecl main_handler (OBJECT *tree, int obj)
{
DEBUG();
switch(obj)
	{
	case FCONNECT:
		connect_to();
		break;
		
	case FDISCONN:
		disconnect();
		break;

	case SEND:
		send_files(TRUE);
		break;
		
	case RECEIVE:
		receive_files(TRUE);
		break;

	case LDIRINFO:
		if (local_dirsel())
			GetLocaldir();
		break;
		
	case LINFO:
		ShowFileInfo(localfiles, local_filecnt);
		break;

	case RINFO:
		if (ftp_socket)
			ShowFileInfo(ftpfiles, ftp_filecnt);
		break;
		
	case LREN:
		RenameLocals();
		break;
		
	case RREN:
		if (ftp_socket)
			RenameRemote();
		break;
		
	case LRD:
		RemoveLocals(1);
		break;
		
	case RRD:
		if (ftp_socket)
			RemoveRemote(1);
		break;

	case LDEL:
		RemoveLocals(0);
		break;
		
	case RDEL:
		if (ftp_socket)
			RemoveRemote(0);
		break;

	case LREF:
		GetLocaldir();
		break;

	case RREF:
		if (ftp_socket)
			{
			ReadDirectory();
			SetInfo(done);
			}
		break;

	case LVIEW:
		view_locals();
		break;
		
	case RVIEW:
		if (ftp_socket)
			view_remotes();
		break;
													
	case LMD:
		if (GetNewFolderName())
			{
			MakeLocalDir();
			GetLocaldir();
			}
		break;
		
	case RMD:
		if (ftp_socket)
			if (GetNewFolderName())
				if (ftp_mkdir(ftp_socket, name_buffer))
					{
					ReadDirectory();
					SetInfo(done);
					}
		break;
				
	case LUP:
		if (local_filetop > 0)
			{
			local_filetop--;
			redraw_object(maindial, LDIR);
			set_dirslider(TRUE, TRUE);
			delay(50);
			}
		break;
		
	case LDOWN:
		if (local_filetop < local_filecnt - maindial[LDIR].ob_height / tx_height)
			{
			local_filetop++;
			redraw_object(maindial, LDIR);
			set_dirslider(TRUE, TRUE);
			delay(50);
			}
		break;

	case LSLD:
		{
		int ox, oy;
		
		objc_offset(maindial, LSLDB, &ox, &oy);
		if (evnt.y > oy)
			{
			local_filetop += (maindial[LDIR].ob_height + tx_height - 1) /
											 tx_height;
			if (local_filetop > local_filecnt - maindial[LDIR].ob_height / tx_height) 
				local_filetop = local_filecnt - maindial[LDIR].ob_height / tx_height;
			redraw_object(maindial, LDIR);
			set_dirslider(TRUE, TRUE);
			delay(50);
			}
		else
			{
			local_filetop -= (maindial[LDIR].ob_height + tx_height - 1) /
											 tx_height;
			if (local_filetop < 0) local_filetop = 0;
			redraw_object(maindial, LDIR);
			set_dirslider(TRUE, TRUE);
			delay(50);
			}
		break;
		}
		
	case LSLDB:
		{
		long slid;
		
		slid = graf_slidebox(maindial, LSLD, LSLDB, 1);
		local_filetop = (int)((long)(local_filecnt - maindial[LDIR].ob_height / tx_height)
													* slid / 1000);
		redraw_object(maindial, LDIR);
		set_dirslider(TRUE, TRUE);
		break;
		}
		
	case LLEFT:
		if (local_left > 0)
			{
			local_left--;
			redraw_object(maindial, LDIR);
			set_dirhslider(TRUE, TRUE);
			delay(50);
			}
		break;
		
	case LRIGHT:
		if (local_left < calc_hwidth(ftp_opts.local_bits, ftp_opts.local_sizes) - maindial[LDIR].ob_width / tx_width)
			{
			local_left++;
			redraw_object(maindial, LDIR);
			set_dirhslider(TRUE, TRUE);
			delay(50);
			}
		break;

	case LSLDH:
		{
		int ox, oy;
		
		objc_offset(maindial, LSLDBH, &ox, &oy);
		if (evnt.x > ox)
			{
			int size = calc_hwidth(ftp_opts.local_bits, ftp_opts.local_sizes);
			
			local_left += (maindial[LDIR].ob_width + tx_width - 1) / tx_width;
			if (local_left > size - maindial[LDIR].ob_width / tx_width) 
				local_left = size - maindial[LDIR].ob_width / tx_width;
			redraw_object(maindial, LDIR);
			set_dirhslider(TRUE, TRUE);
			delay(50);
			}
		else
			{
			local_left -= (maindial[LDIR].ob_width + tx_width - 1) / tx_width;
			if (local_left < 0) local_left = 0;
			redraw_object(maindial, LDIR);
			set_dirhslider(TRUE, TRUE);
			delay(50);
			}
		break;
		}
		
	case LSLDBH:
		{
		long slid;
		
		slid = graf_slidebox(maindial, LSLDH, LSLDBH, 0);
		local_left = (int)((long)(calc_hwidth(ftp_opts.local_bits, ftp_opts.local_sizes) - maindial[LDIR].ob_width / tx_width)
													* slid / 1000);
		redraw_object(maindial, LDIR);
		set_dirhslider(TRUE, TRUE);
		break;
		}

	case RUP:
		if (ftp_filetop > 0)
			{
			ftp_filetop--;
			redraw_object(maindial, RDIR);
			set_dirslider(FALSE, TRUE);
			delay(50);
			}
		break;
		
	case RDOWN:
		if (ftp_filetop < ftp_filecnt - maindial[RDIR].ob_height / tx_height)
			{
			ftp_filetop++;
			redraw_object(maindial, RDIR);
			set_dirslider(FALSE, TRUE);
			delay(50);
			}
		break;

	case RSLD:
		{
		int ox, oy;
		
		objc_offset(maindial, RSLDB, &ox, &oy);
		if (evnt.y > oy)
			{
			ftp_filetop += (maindial[RDIR].ob_height + tx_height - 1) /
										 tx_height;
			if (ftp_filetop > ftp_filecnt - maindial[RDIR].ob_height / tx_height) 
				ftp_filetop = ftp_filecnt - maindial[RDIR].ob_height / tx_height;
			redraw_object(maindial, RDIR);
			set_dirslider(FALSE, TRUE);
			delay(50);
			}
		else
			{
			ftp_filetop -= (maindial[RDIR].ob_height + tx_height - 1) /
											 tx_height;
			if (ftp_filetop < 0) ftp_filetop = 0;
			redraw_object(maindial, RDIR);
			set_dirslider(FALSE, TRUE);
			delay(50);
			}
		break;
		}

	case RSLDB:
		{
		long slid;
		
		slid = graf_slidebox(maindial, RSLD, RSLDB, 1);
		ftp_filetop = (int)((long)(ftp_filecnt - maindial[RDIR].ob_height / tx_height)
															* slid / 1000);
		redraw_object(maindial, RDIR);
		set_dirslider(FALSE, TRUE);
		break;
		}
							
	case RLEFT:
		if (ftp_left > 0)
			{
			ftp_left--;
			redraw_object(maindial, RDIR);
			set_dirhslider(FALSE, TRUE);
			delay(50);
			}
		break;
		
	case RRIGHT:
		if (ftp_left < calc_hwidth(ftp_opts.ftp_bits, ftp_opts.ftp_sizes) - maindial[RDIR].ob_width / tx_width)
			{
			ftp_left++;
			redraw_object(maindial, RDIR);
			set_dirhslider(FALSE, TRUE);
			delay(50);
			}
		break;

	case RSLDH:
		{
		int ox, oy;
		
		objc_offset(maindial, RSLDBH, &ox, &oy);
		if (evnt.x > ox)
			{
			int size = calc_hwidth(ftp_opts.ftp_bits, ftp_opts.ftp_sizes);
			
			ftp_left += (maindial[RDIR].ob_width + tx_width - 1) / tx_width;
			if (ftp_left > size - maindial[RDIR].ob_width / tx_width) 
				ftp_left = size - maindial[RDIR].ob_width / tx_width;
			redraw_object(maindial, RDIR);
			set_dirhslider(FALSE, TRUE);
			delay(50);
			}
		else
			{
			ftp_left -= (maindial[RDIR].ob_width + tx_width - 1) / tx_width;
			if (ftp_left < 0) ftp_left = 0;
			redraw_object(maindial, RDIR);
			set_dirhslider(FALSE, TRUE);
			delay(50);
			}
		break;
		}
		
	case RSLDBH:
		{
		long slid;
		
		slid = graf_slidebox(maindial, RSLDH, RSLDBH, 0);
		ftp_left = (int)((long)(calc_hwidth(ftp_opts.ftp_bits, ftp_opts.ftp_sizes) - maindial[RDIR].ob_width / tx_width)
													* slid / 1000);
		redraw_object(maindial, RDIR);
		set_dirhslider(FALSE, TRUE);
		break;
		}

	case LDIR|0x8000:
		mark_file(obj);
		set_buttonstate();
		break;
		
	case RDIR|0x8000:
		if (ftp_socket)
			{
			mark_file(obj);
			set_buttonstate();
			}
		break;
		
	case LDIR:
		mark_file(obj);
		set_buttonstate();
		break;
							
	case RDIR:
		if (ftp_socket)
			{
			mark_file(obj);
			set_buttonstate();
			}
		break;
	
	case MAINSIZER:
		{
		int newwidth, newheight;
		
		graf_rubbox(tree->ob_x, tree->ob_y, min_dialwidth, min_dialheight, &newwidth, &newheight);
		resize_main(newwidth, newheight);
		break;
		}

	case 0x2000|WM_FULLED:
		{
		GRECT cr, fr;
		int wnd = get_dialog_wind(tree);
		
		wind_get_grect(wnd, WF_CURRXYWH, &cr);
		wind_get_grect(ROOT, WF_WORKXYWH, &fr);
		if (cr.g_x == fr.g_x && cr.g_y == fr.g_y && cr.g_w == fr.g_w && cr.g_h == fr.g_h)
			wind_get_grect(wnd, WF_PREVXYWH, &fr);
			
		wind_calc(WC_WORK, NAME, &fr, &cr);
		
		tree->ob_x = cr.g_x;
		tree->ob_y = cr.g_y;		
		resize_main(cr.g_w, cr.g_h);
		break;					
		}
				
	case 0x2000|WM_CLOSED:
		exit_flg = TRUE;
		break;
	}
}

/* -------------------------
   | Main routine of STTCP |
   ------------------------- */
void main(void)
{
int bclicks;
#ifdef TRAP_LOG
	long oldbus, oldaddr;
	void trapbus( void );
	void trapaddr( void );
#endif

#ifdef TRAP_LOG
{
long old_stk;
extern jmp_buf trapcont;
old_stk = Super(0L);
oldbus = *(long *)0x8;
oldaddr = *(long *)0xC;

*(long *)0x8 = (long)trapbus;
*(long *)0xC = (long)trapaddr;
Super((void *)old_stk);
}
#endif

DEBUG();
appl_init();
open_vwork();

if (init_rsc())
	{
/*	if (CheckSystem())*/
		Pdomain(1);

	naes = check_cookie('nAES');

	get_bootdrive();
	read_installdir();

	if (init_dialogs() < 0)
		{
		form_alert(1, "[1][Dialog initialization failed!][Ok]");
		goto end;
		}
		
	Dgetpath(local_dir + 2, 0);
	local_dir[1] = ':';
	local_dir[0] = Dgetdrv() + 'A';

	load_options();
	init_colorcache();
	open_intro();

	load_ftpcollections();
	load_my_images();
	
	close_intro();
	
	graf_mouse(ARROW, 0L);
	
	rsrc_gaddr(R_TREE, FTPMENU, &menu_adr);
	menu_bar(menu_adr, TRUE);

	set_menu();
	
#ifdef WAIT_ABOUT
	about_info(WAIT_FLG);
#endif
	
	rsrc_gaddr(R_TREE, DFTP, &maindial);
	
	SetInfo("");
	maindial[RDIR].ob_type = G_USERDEF;
	maindial[RDIR].ob_spec.userblk = &rdirv_ublk;
	maindial[LDIR].ob_type = G_USERDEF;
	maindial[LDIR].ob_spec.userblk = &ldirv_ublk;
	
	init_grfbuttons();

	maindial[RDIRINFO].ob_spec.tedinfo->te_ptext[0] =
	maindial[LDIRINFO].ob_spec.tedinfo->te_ptext[0] = 0;

	align_slider_objects(maindial, LSLDBKG, LUP, LSLD, LSLDB, LDOWN);
	align_slider_objects(maindial, LSLDBOT, LLEFT, LSLDH, LSLDBH, LRIGHT);
	align_slider_objects(maindial, RSLDBKG, RUP, RSLD, RSLDB, RDOWN);
	align_slider_objects(maindial, RSLDBOT, RLEFT, RSLDH, RSLDBH, RRIGHT);
	
	set_slider_objects(maindial, LSLD, LSLDB, 0, 0,
										 maindial[LDIR].ob_height / tx_height, FALSE);
	set_slider_objects(maindial, LSLDH, LSLDBH, 0, 0,
										 maindial[LDIR].ob_width / tx_width, FALSE);
	set_slider_objects(maindial, RSLD, RSLDB, 0, 0,
										 maindial[RDIR].ob_height / tx_height, FALSE);
	set_slider_objects(maindial, RSLDH, RSLDBH, 0, 0,
										 maindial[RDIR].ob_width / tx_width, FALSE);
		
	open_kinddialog(maindial, maindial[MAIN_TITLE].ob_spec.free_string, iconified, SMALLER|NAME|MOVER|CLOSER|FULLER);
	dlgset_help_info(maindial, helpfile, mainhelp_ids, "MAIN_");

	exit_flg = FALSE;

	min_dialwidth = maindial->ob_width;
	min_dialheight = maindial->ob_height;
	init_mainsizer();

	if (main_x != -1)
		{
		GRECT d, m;
		wind_get_grect(ROOT, WF_WORKXYWH, &d);
		
		if (main_x + ftp_opts.main_w > d.g_x + d.g_w)
			{
			ftp_opts.main_w = d.g_w + d.g_x - main_x;
			if (ftp_opts.main_w < min_dialwidth)
				{
				main_x -= min_dialwidth - ftp_opts.main_w;
				ftp_opts.main_w = min_dialwidth;
				}
			}
		if (main_y + ftp_opts.main_h > d.g_y + d.g_h)
			{
			ftp_opts.main_h = d.g_h + d.g_y - main_y;
			if (ftp_opts.main_h < min_dialheight)
				{
				main_y -= min_dialheight - ftp_opts.main_h;
				ftp_opts.main_h = min_dialheight;
				}
			}

		m.g_x = main_x;		m.g_y = main_y;		
		m.g_w = ftp_opts.main_w;		
		m.g_h = ftp_opts.main_h;
		
		wind_calc(WC_BORDER, NAME, &m, &d);
		msg[0] = WM_MOVED;
		msg[3] = get_dialog_wind(maindial);
		msg[4] = d.g_x;
		msg[5] = d.g_y;
		msg[6] = d.g_w;
		msg[7] = d.g_h;
		dialog_message(msg);
		form_dial(FMD_FINISH, &empty, (GRECT *)&msg[4]);
		
		if (ftp_opts.main_w > 0)
			resize_main(ftp_opts.main_w, ftp_opts.main_h);
		}

	set_active(maindial, main_handler, TRUE, 0);
	GetLocaldir();	
		
	if (ftp_opts.show_conn)
		connect_to();

#ifdef TRAP_LOG
	if (!setjmp(trapcont))
#endif	
		do
			{
	 	  int bsk;
	 	  
			if (!naes)
				{
				graf_mkstate( &evnt);
				bsk = evnt.bstate & 2;
				if ( !(bsk) )
		  		bclicks = 256+2;
				else
				  bclicks = 0;
				 
				bsk = 0;
				}
			else
				{
				evnt_multi(MU_TIMER,
									 258, 3, 0,
									 0, &empty,
									 0, &empty,
									 msg,
									 1L,
									 &evnt, &k_return, &clicks);
				bsk = evnt.bstate & 2;
				bclicks = 256+2;
				}
	
			which = evnt_multi(MU_MESAG|MU_TIMER|MU_KEYBD|MU_BUTTON,
												 bclicks, 3, bsk,
												 0, &empty,
												 0, &empty,
												 msg,
												 200L,
												 &evnt, &k_return, &clicks);
	
			which = evnt_handler(which, msg, &evnt, k_return, clicks);
			app_callback(which, msg, &evnt, k_return, clicks);
									 						
			}while(!exit_flg);
		
	disconnect();

#ifndef EVAL
	save_options();
#endif

	close_dialog(maindial, TRUE);
	
#ifdef WAIT_ABOUT2
	about_info(WAIT_FLG);
#endif

	exit_dialogs();
	menu_bar(menu_adr, FALSE);
	}
	
end:

#ifdef TRAP_LOG
{
long old_stk;

old_stk = Super(0L);
*(long *)0x8 = oldbus;
*(long *)0xC = oldaddr;
Super((void *)old_stk);
}
#endif

cls_vwork();
appl_exit();
}

/* -------------------------------------------------------------
   | Application call for handling events during modal dialogs |
   ------------------------------------------------------------- */
void cdecl app_callback( int awhich, int *amsg, EVNTDATA *ev, int kr, int br )
{
DEBUG();
which = awhich;
memcpy(&evnt, ev, sizeof(EVNTDATA));
memcpy(msg, amsg, sizeof(msg));
k_return = kr;
clicks = br;

if (which & MU_KEYBD)
	{
	int top;
	
	wind_get_int(ROOT, WF_TOP, &top);
	menu_keys();
	}

if ((which & MU_BUTTON) && clicks && (evnt.bstate & 2))
	{
	int obj;

	obj = objc_find(maindial, ROOT, MAX_DEPTH, evnt.x, evnt.y);
	
	if (obj == LDIR)
		local_popup();
	else if (obj == RDIR)
		remote_popup();
	}
		
if (which & MU_MESAG)
	{
	if (is_view(msg[3]))
		{
		view_msg(msg);
		set_menu();
		}
	message_handler();
	}
}

/* ------------------------
   | handle message queue |
   ------------------------ */
void message_handler(void)
{
DEBUG();
switch(msg[0])
	{
	case AP_TERM:
		exit_flg = TRUE;
		break;

/* Dialer message */
	case 0x5810:
		connect_now();
		set_buttonstate();
		break;

#ifndef EVAL
																	/* FTP - Messages: */
	case 0x3210:											/* Open Connection */
		AppConnect(msg);
		break;

	case 0x3211:											/* Connect host/user/password */
		AppConnectS(msg);
		break;
		
	case 0x3212:											/* Cd path */
		AppCd(msg);
		break;

	case 0x3213:											/* Get remotefile localfile */
		AppGet(msg);
		break;

	case 0x3214:											/* Put localfile [remotefile] */
		AppPut(msg);
		break;
		
	case 0x3215:											/* Set type */
		AppType(msg);
		break;
		
	case 0x3216:											/* Close */
		AppClose(msg);
		break;

	case 0x3217:
		/* AppDirFirst(); */
		break;
		
	case 0x3218:
		/* AppDirNext(); */
		break;
#endif

	case MN_SELECTED:
		menu_action(msg);
		break;

	case WM_NEWTOP:	
	case WM_TOPPED:
		wind_set_int(msg[3], WF_TOP, 0);
		break;

	case WM_FULLED:
	  {
		if (is_view(msg[3]))
			{
			int cx, cy, cw, ch;
			int fx, fy, fw, fh;
			
			wind_get(msg[3], WF_CURRXYWH, &cx, &cy, &cw, &ch);
			wind_get(msg[3], WF_FULLXYWH, &fx, &fy, &fw, &fh);
			if (cx == fx && cy == fy && cw == fw && ch == fh)
				wind_get(msg[3], WF_PREVXYWH, &fx, &fy, &fw, &fh);
				
			wind_set(msg[3], WF_CURRXYWH, fx, fy, fw, fh);
			}
		break;					
		}
		
	case WM_SIZED:
	case WM_MOVED:
		wind_set(msg[3], WF_CURRXYWH, msg[4], msg[5], msg[6], msg[7]);
		break;

	default:
		break;
	}
}

/* ----------------
   | Menu handler |
   ---------------- */
void menu_action(int *msg)
{
int title;

DEBUG();
title = msg[3];
switch(msg[4])
	{
	case M_ABOUT:
		about_info();
		break;

	case M_OPENLOG:
		view_file(ftp_opts.logfile, FALSE, logfilelabel);
		break;
		
	case M_EXIT:
		exit_flg = TRUE;
		break;

	case M_COPY:
		view_copy();
		break;

	case M_L_SIZE:
		ftp_opts.local_bits ^= DIR_SIZE;
		set_menu();
		redraw_object(maindial, LDIR);
		set_dirhslider(TRUE, TRUE);
		break;
		
	case M_L_DATE:
		ftp_opts.local_bits ^= DIR_DATE;
		set_menu();
		redraw_object(maindial, LDIR);
		set_dirhslider(TRUE, TRUE);
		break;
		
	case M_L_ATTR:
		ftp_opts.local_bits ^= DIR_ATTR;
		set_menu();
		redraw_object(maindial, LDIR);
		set_dirhslider(TRUE, TRUE);
		break;

	case M_R_SIZE:
		ftp_opts.ftp_bits ^= DIR_SIZE;
		set_menu();
		redraw_object(maindial, RDIR);
		set_dirhslider(FALSE, TRUE);
		break;
		
	case M_R_DATE:
		ftp_opts.ftp_bits ^= DIR_DATE;
		set_menu();
		redraw_object(maindial, RDIR);
		set_dirhslider(FALSE, TRUE);
		break;

	case M_R_USER:
		ftp_opts.ftp_bits ^= DIR_USER;
		set_menu();
		redraw_object(maindial, RDIR);
		set_dirhslider(FALSE, TRUE);
		break;

	case M_R_GROUP:
		ftp_opts.ftp_bits ^= DIR_GROUP;
		set_menu();
		redraw_object(maindial, RDIR);
		set_dirhslider(FALSE, TRUE);
		break;
		
	case M_R_ATTR:
		ftp_opts.ftp_bits ^= DIR_ATTR;
		set_menu();
		redraw_object(maindial, RDIR);
		set_dirhslider(FALSE, TRUE);
		break;
		
	case M_VASCII:
		set_ascii_mode();
		break;
				
	case M_VBIN:
		set_binary_mode();
		break;
				
	case M_SETOPT:
		set_options();
		break;

	case M_SAVOPT:
#ifdef EVAL
		not_demo();
#else
		save_options();
#endif
		break;
	}
menu_tnormal(menu_adr, title, TRUE);
}

/* --------------------------------
   | Handle menu key combinations |
   -------------------------------- */
int menu_keys(void)
{
DEBUG();
switch(k_return & 0xFF)
	{
	case 1:
		set_ascii_mode();
		break;
		
	case 2:
		set_binary_mode();
		break;

	case 3:
		view_copy();
		break;
				
	case 9:
		about_info();
		return TRUE;
		
	case 0x5:
		set_options();
		return TRUE;
		
	case 0xF:
		view_file(ftp_opts.logfile, FALSE, logfilelabel);
		return TRUE;
		
	case 0x11:
		exit_flg = TRUE;
		return TRUE;
	}
return FALSE;
}

/* ------------
   | Set menu |
   ------------ */
void set_menu(void)
{
int top;

DEBUG();
wind_get_int(ROOT, WF_TOP, &top);

menu_ienable(menu_adr, M_OPENLOG, ftp_opts.logfile[0] != 0);

menu_ienable(menu_adr, M_VASCII, is_view(top));
menu_ienable(menu_adr, M_VBIN, 	 is_view(top));
menu_ienable(menu_adr, M_COPY, 	 FALSE);

menu_icheck(menu_adr, M_L_SIZE,  ftp_opts.local_bits & DIR_SIZE);
menu_icheck(menu_adr, M_L_DATE,  ftp_opts.local_bits & DIR_DATE);
menu_icheck(menu_adr, M_L_ATTR,  ftp_opts.local_bits & DIR_ATTR);

menu_icheck(menu_adr, M_R_SIZE,  ftp_opts.ftp_bits & DIR_SIZE);
menu_icheck(menu_adr, M_R_DATE,  ftp_opts.ftp_bits & DIR_DATE);
menu_icheck(menu_adr, M_R_USER,  ftp_opts.ftp_bits & DIR_USER);
menu_icheck(menu_adr, M_R_GROUP, ftp_opts.ftp_bits & DIR_GROUP);
menu_icheck(menu_adr, M_R_ATTR,  ftp_opts.ftp_bits & DIR_ATTR);

if (is_view(top))	set_view_menu();
}
