/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

int min(int a, int b)	{	return ((a < b) ? a : b);	}
int max(int a, int b)	{	return ((a > b) ? a : b);	}

#define MagX_COOKIE 'MagX'
#define MiNT_COOKIE 'MiNT'

/* -----------------------------
   | Check for cookie presence |
   ----------------------------- */
long check_cookie(long cook)
{
long old_stk, *old_jar;

DEBUG();
old_stk = Super(0L);

old_jar = *(long **)0x5a0;
Super((void *)old_stk);

if (!old_jar)
	return 0L;
	
while(*old_jar)
	{
	if (*old_jar == cook)
		return old_jar[1];
		
	old_jar += 2;
	}

return 0L;
}

/* ------------------------
   | Check current system |
   ------------------------ */
int CheckSystem(void)
{
int sys;

DEBUG();
sys = 0;
if (check_cookie(MiNT_COOKIE))
	{
  if (_GemParBlk.global[1] > 1 || _GemParBlk.global[1] == -1)
    sys = 3;
  else
  	sys = 1;
	}

if (check_cookie(MagX_COOKIE))
  sys = 2;
  
return sys;
}

/* ------------------------------------
   | Calculate rectangle intersection |
   ------------------------------------ */
int rc_intersect(GRECT *s1, GRECT *s2)
{
int tx, ty, tw, th;

DEBUG();
tw = min(s2->g_x + s2->g_w, s1->g_x + s1->g_w);
th = min(s2->g_y + s2->g_h, s1->g_y + s1->g_h);
tx = s2->g_x = max(s2->g_x, s1->g_x);
ty = s2->g_y = max(s2->g_y, s1->g_y);
s2->g_w = tw - tx;
s2->g_h = th - ty;

return ((tw > tx) && (th > ty));
}

/* -----------------------
   | Set new info string |
   ----------------------- */
void SetInfo(char *newinfo)
{
DEBUG();
maindial[INFOC].ob_spec.free_string = newinfo;
redraw_object(maindial, INFOB);
}

/* -----------------------
   | Get new folder name |
   ----------------------- */
int GetNewFolderName(void)
{
OBJECT *ctree;
int obj;
static int nf_ids[] = {NEWNAME, NEWF_OK, NEWF_CNC, 0};

DEBUG();
rsrc_gaddr(R_TREE, NEWFOLD, &ctree);
ctree[NEWNAME].ob_spec.tedinfo->te_ptext[0] = 0;

open_kinddialog(ctree, ctree[NEWFOLD_TITLE].ob_spec.free_string, iconified, SMALLER|NAME|MOVER|CLOSER);
dlgset_help_info(ctree, helpfile, nf_ids, "NF_");
obj = do_dialog(ctree, NEWNAME, app_callback);
close_dialog(ctree, TRUE);

strcpy(name_buffer, ctree[NEWNAME].ob_spec.tedinfo->te_ptext);
return (obj == NEWF_OK);
}

/* -----------------------
   | Get new folder name |
   ----------------------- */
int GetRenamedName(char *oldname)
{
OBJECT *ctree;
int obj;
static int ren_ids[] = {REN_NEW, REN_OK, REN_CNC, 0};

DEBUG();
rsrc_gaddr(R_TREE, RENAME, &ctree);

strncpy(ctree[REN_OLD].ob_spec.free_string, oldname, 30);
ctree[REN_OLD].ob_spec.free_string[30] = 0;

strcpy(ctree[REN_NEW].ob_spec.tedinfo->te_ptext,
			 ctree[REN_OLD].ob_spec.free_string);
			 
open_kinddialog(ctree, ctree[RENAME_TITLE].ob_spec.free_string, iconified, SMALLER|NAME|MOVER|CLOSER);
dlgset_help_info(ctree, helpfile, ren_ids, "REN_");
obj = do_dialog(ctree, REN_NEW, app_callback);
close_dialog(ctree, TRUE);

strcpy(name_buffer, ctree[REN_NEW].ob_spec.tedinfo->te_ptext);
return (obj == REN_OK);
}


/* -------------------------------------
   | Check current state of connection |
   ------------------------------------- */
int check_connection(int function)
{
DEBUG();
switch (get_connected())
	{
	case -1:
		form_alert(1, nodriver_msg);
		return FALSE;
	
	case 0:
		{
		int draconis_id;
		
		draconis_id = appl_find("DRACONIS");
		if (draconis_id < 0)
			form_alert(1, offline_msg);
		else
			{
			int msg[8];
			
			msg[0] = 0x5811;
			msg[1] = gl_apid;
			msg[3] = function;
			appl_write(draconis_id, 16, msg);
			}
		return FALSE;
		}
		
	case 1:
		while (get_connected() == 1)
		  if (get_connected() != 2 && get_connected() != 1)
			  return FALSE;
			
	case 2:
		return TRUE;
	}
return FALSE;
}

/* --------------------------
   | Wait for mouse release |
   -------------------------- */
void wait_mouse_release(void)
{
DEBUG();
check_mouse();
/*graf_mkstate( &evnt);*/

if (evnt.bstate & 1)
	evnt_button (1, evnt.bstate & 1, 0x00, &evnt);		/* Warte auf Mausknopf */
}

#ifdef EVAL
/* ----------------------------
   | Not in the demo version! |
   ---------------------------- */
void not_demo(void)
{
DEBUG();
open_dialog(notdemo, NULL, iconified, SMALLER|NAME|MOVER|CLOSER);
do_dialog(notdemo, 0, app_callback);
close_dialog();
}
#endif

/* ----------------------------
   | Check mouse button state |
   ---------------------------- */
void check_mouse(void)
{
DEBUG();
which = evnt_multi(MU_BUTTON|MU_TIMER|MU_MESAG,
					 1, 1, 1,
					 0, &empty,
					 0, &empty,
					 msg,
					 50L,
					 &evnt,
					 &k_return, &clicks);

if (!(which & MU_BUTTON) && !evnt.bstate)
	{
	graf_mkstate(&evnt);
	if (evnt.bstate)
		{
		which |= MU_BUTTON;
		clicks = 1;
		}
	}

which = evnt_handler(which & ~MU_BUTTON, msg, &evnt, k_return, clicks);
}

/* ----------------------------
   | Check mouse button state |
   ---------------------------- */
void check_events(void)
{
int bsk, bclicks;
  
DEBUG();
if (!naes)
	{
	graf_mkstate( &evnt);
	bsk = evnt.bstate & 3;
	if ( !(bsk) )
		bclicks = 256+2;
	else
	  bclicks = 0;
	 
	bsk = 0;
	}
else
	{
	evnt_multi(MU_TIMER,
						 258, 3, 0,
						 0, &empty,
						 0, &empty,
						 msg,
						 1L,
						 &evnt, &k_return, &clicks);
	bsk = evnt.bstate & 3;
	bclicks = 256+2;
	}
			
which = evnt_multi(MU_BUTTON|MU_TIMER|MU_KEYBD|MU_MESAG,
					 bclicks, 3, bsk,
					 0, &empty,
					 0, &empty,
					 msg,
					 1L,
					 &evnt,
					 &k_return, &clicks);

which = evnt_handler(which, msg, &evnt, k_return, clicks);
}
