/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"
#include "..\include\color.h"

#define FTP_VERSION 0x00020000L

static int copy_mode;
static int r_mode[] = {AUTOMODE, ASCIMODE, BINMODE};

int bps_mode[] = {BITSSECS, BYTESECS};
int l_sort[] = {L_NAME, L_EXT, L_DATE, L_SIZE};
int r_sort[] = {R_NAME, R_EXT, R_DATE, R_SIZE};
int cnve_ids[] = {CNV_E1, 	CNV_E2, 	CNV_E3, 	CNV_E4, 	CNV_E5,
 								  CNV_E6, 	CNV_E7, 	CNV_E8, 	CNV_E9, 	CNV_E10,
									CNV_E11, CNV_E12, 	CNV_E13, 	CNV_E14, 	CNV_E15,
									CNV_E16, CNV_E17, 	CNV_E18, 	CNV_E19, 	CNV_E20,
									CNV_E21};
int cnvd_ids[] = {CNV_D1, 	CNV_D2, 	CNV_D3, 	CNV_D4, 	CNV_D5,
									CNV_D6, 	CNV_D7, 	CNV_D8, 	CNV_D9, 	CNV_D10,
									CNV_D11, CNV_D12, 	CNV_D13, 	CNV_D14, 	CNV_D15,
									CNV_D16, CNV_D17, 	CNV_D18, 	CNV_D19, 	CNV_D20,
									CNV_D21};

int ascii_ids[] = {ASC01, ASC02, 	ASC03, 	ASC04, 	ASC05,
									 ASC06, ASC07, 	ASC08, 	ASC09, 	ASC10,
									 ASC11, ASC12, 	ASC13, 	ASC14, 	ASC15,
									 ASC16, ASC17, 	ASC18, 	ASC19, 	ASC20,
									 ASC21, ASC22, 	ASC23, 	ASC24, 	ASC25,
									 ASC26, ASC27, 	ASC28, 	ASC29, 	ASC30,
									 ASC31, ASC32, 	ASC33, 	ASC34, 	ASC35,
									 ASC36, ASC37, 	ASC38, 	ASC39, 	ASC40,
									 ASC41, ASC42, 	ASC43, 	ASC44, 	ASC45,
									 ASC46, ASC47, 	ASC48, 	ASC49, 	ASC50,
									 ASC51, ASC52, 	ASC53, 	ASC54, 	ASC55,
									 ASC56, ASC57, 	ASC58, 	ASC59, 	ASC60,
									 ASC61, ASC62, 	ASC63, 	ASC64, 	ASC65,
									 ASC66, ASC67, 	ASC68, 	ASC69, 	ASC70,
									 ASC71, ASC72
									 };
int case_ids[] = {FILESMIX, FILESUP, FILESLOW};

static int help_ids[] = {
												O_SAVE, OPT_OK, OPT_CNC, 				/* 0-2 */
												OPFTYP, OPLOG, OPEXT, OPVIEW, 
												OPCOPY,	OPGEN,									/* 3-8 */

												V_DELETE, V_OVERWR, TABSIZE,
												SV_WINDW, ICONCACHE, BUBBLEGEM,
												ICONS, ICONDROP,								/* 9-16 */												  

												SHOWCONN, PRESTIME, BITSSECS,
			 									BYTESECS, IN_BUF,	OUT_BUF,
												ANOEMAIL, EXT_COPY,							/* 17-24 */

												L_NAME, L_EXT, L_DATE, L_SIZE,
												R_NAME, R_EXT, R_DATE, R_SIZE,
												SAME_COLOUR, NEWER_COLOUR,
												OLDER_COLOUR, NONEXIST_COLOUR,
												COLOUR_FILES, V_LINKS,					/* 25-38  */
												
												ENABECNV,	CONV_LIST,						/* 39 -40 */
 												ENABLOG,
 												LOGCMDS, LOGERROR, LOGDATA,
											  LOGFILE,												/* 41-45 */
												
												ASCII_LIST,											/* 46-46 */
												
												0};
												
static int helpinitialized = FALSE;

/* -------------------
   | Set option page |
   ------------------- */
void SetToOpPage(OBJECT *tree, int page, int obj)
{
static int tabids[] = {OPFTYP, OPLOG, OPEXT, OPVIEW, OPCOPY, OPGEN};
int i;

DEBUG();
if (tree[obj].ob_flags & SELECTABLE)
	{
	BindTab(tree, OTABBKG, page);
	redraw_object(tree, OTABBKG);
	
	for (i = 0; i < 6; i++)
		{
		tree[tabids[i]].ob_flags |= SELECTABLE|EXIT;
		SetSelect(tree, tabids[i], obj == tabids[i]);
		redraw_object(tree, tabids[i]);
		}
	tree[obj].ob_flags &= ~(SELECTABLE|EXIT);
	redraw_object(tree, obj);
	}
}

/* ----------------------
   | Manage Option tree |
   ---------------------- */
void set_options(void)
{
char *cpy;
int obj;
int old_lsort, old_rsort;

DEBUG();
old_lsort = ftp_opts.l_sort;
old_rsort = ftp_opts.r_sort;

#ifdef EVAL
	DisableObj(option_dial, O_SAVE);
#endif

if (!helpinitialized)
	{
	int i;

	for (i =  9; i <= 16; i++)	help_ids[i] += option_tabs[0];		
	for (i = 17; i <= 24; i++)	help_ids[i] += option_tabs[1];		
	for (i = 25; i <= 38; i++)	help_ids[i] += option_tabs[2];		
	for (i = 39; i <= 40; i++)	help_ids[i] += option_tabs[3];		
	for (i = 41; i <= 45; i++)	help_ids[i] += option_tabs[4];		
	for (i = 46; i <= 46; i++)	help_ids[i] += option_tabs[5];		
	
	helpinitialized = TRUE;
	}

cpy = option_dial[option_tabs[0] + ICONS].ob_spec.tedinfo->te_ptext;
strncpy(cpy, getCollectionName(), 20);
cpy[20] = 0;

SetToOpPage(option_dial, option_tabs[0], OPGEN);
open_kinddialog(option_dial, option_dial[OPTIONS_TITLE].ob_spec.free_string, iconified, SMALLER|NAME|MOVER|CLOSER);
dlgset_help_info(option_dial, helpfile, help_ids, "OPT_");
do
	{
	obj = do_dialog(option_dial, 0, app_callback);
	switch(obj)
		{
		case OPGEN:			SetToOpPage(option_dial, option_tabs[0], OPGEN);	break;
		case OPCOPY:		SetToOpPage(option_dial, option_tabs[1], OPCOPY);	break;
		case OPVIEW:		SetToOpPage(option_dial, option_tabs[2], OPVIEW);	break;
		case OPEXT:			SetToOpPage(option_dial, option_tabs[3], OPEXT);	break;
		case OPLOG:			SetToOpPage(option_dial, option_tabs[4], OPLOG);	break;
		case OPFTYP:		SetToOpPage(option_dial, option_tabs[5], OPFTYP);	break;

		case O_SAVE:
			copy_options(TRUE);
			save_options();
			break;

		default:		
			if (IsSelected(option_dial, OPGEN))				/* General Page active ? */
				{
				switch(obj - option_tabs[0])
					{
					case ICONS:
					case ICONDROP:
						{
						int x, y;
						
						objc_offset(option_dial, option_tabs[0] + ICONS, &x, &y);
						select_ftpbuttons(x, y, option_dial[option_tabs[0] + ICONS].ob_width);
						strncpy(cpy, getCollectionName(), 20);
						cpy[20] = 0;
						break;
						}
					}
				}
			else if (IsSelected(option_dial, OPVIEW))	/* Browser Page active ? */
				{
				switch(obj - option_tabs[2])
					{
					case SAME_COLOUR:
					case NEWER_COLOUR:
					case OLDER_COLOUR:
					case NONEXIST_COLOUR:
						{
						int defcol;
						OBJECT *pop;
						
						switch (obj - option_tabs[2])
							{
							case SAME_COLOUR:			defcol = 4; break;
							case NEWER_COLOUR: 		defcol = 3; break;
							case OLDER_COLOUR: 		defcol = 2; break;
							case NONEXIST_COLOUR:	defcol = 1; break;
							}
						
						rsrc_gaddr(R_TREE, CSELPOP, &pop);	
		  			color_popup( option_dial, obj, pop, defcol );
						break;
						}
					}
				}
		}
	}while(obj != OPT_OK && obj != OPT_CNC && obj != (0x2000|WM_CLOSED));
close_dialog(option_dial, TRUE);

copy_options(obj == OPT_OK);

if (old_lsort != ftp_opts.l_sort)
	{
	sort_dir(local_filecnt, localfiles, ftp_opts.l_sort);
	redraw_object(maindial, LDIR);
	}
	
if (old_rsort != ftp_opts.r_sort)
	{
	sort_dir(ftp_filecnt, ftpfiles, ftp_opts.r_sort);
	redraw_object(maindial, RDIR);
	}

/*position_grfbuttons();*/
}

/* ------------------------
   | Save current options |
   ------------------------ */
void save_options(void)
{
long fh;
int error;

DEBUG();
copy_options(1);

fh = Fcreate("drftp.cnf", 0);
if (fh > -1)
	{
	char tmp_password[46];
	long version = FTP_VERSION;
	
	if (!ftp_opts.sv_window)
		main_x = main_y = ftp_opts.main_w = ftp_opts.main_h -1;
	else
		{
		main_x = maindial->ob_x;
		main_y = maindial->ob_y;
		ftp_opts.main_w = maindial->ob_width;
		ftp_opts.main_h = maindial->ob_height;
		}
	
	strcpy(tmp_password, act_conndef.password);
	if (!act_conndef.save_password)
		act_conndef.password[0] = 0;

	error = FALSE;		

	if (Fwrite((int)fh, 4, "DFTP") != 4)
		error = TRUE;

	if (!error)
  	if (Fwrite((int)fh, sizeof(version), &version) != sizeof(version))
	  	error = TRUE;

	if (!error)
  	if (Fwrite((int)fh, sizeof(ftp_opts), &ftp_opts) != sizeof(ftp_opts))
	  	error = TRUE;

	if (!error)
		if (Fwrite((int)fh, sizeof(CONN_INF), &act_conndef) != sizeof(CONN_INF))
			error = TRUE;

	if (!error)
		if (Fwrite((int)fh, sizeof(main_x), &main_x) != sizeof(main_x))
			error = TRUE;

	if (!error)
		if (Fwrite((int)fh, sizeof(main_y), &main_y) != sizeof(main_y))
			error = TRUE;

	if (!error)
		if (Fwrite((int)fh, sizeof(conn_cnt), &conn_cnt) != sizeof(conn_cnt))
			error = TRUE;

	if (!error)
		if (conn_cnt > 0)
			if (Fwrite((int)fh, sizeof(CONN_INF) * conn_cnt, conn_defs) != 
						sizeof(CONN_INF) * conn_cnt)
				error = TRUE;

	if (error)
		form_alert(1, error_write);
			
	strcpy(act_conndef.password, tmp_password);
	Fclose((int)fh);
	}
}

/* --------------------------------------
   | Convert old connection information |
   -------------------------------------- */
void convert_old(OLD_CONN_INF *old_conn, CONN_INF *conn)
{
DEBUG();
conn->anonymous 				= old_conn->anonymous;
conn->save_password 		= old_conn->save_password;
conn->max_retries 			= old_conn->max_retries;
conn->timeout_secs 			= old_conn->timeout_secs;
conn->port_number 			= old_conn->port_number;
conn->passive_transfer	= old_conn->passive_transfer;
conn->show_dots					= old_conn->show_dots;
conn->case_mode					= old_conn->case_mode;

strcpy(conn->connection, old_conn->connection);
strcpy(conn->host, 			 old_conn->host);
strcpy(conn->user, 			 old_conn->user);
strcpy(conn->password, 	 old_conn->password);
strcpy(conn->account, 	 old_conn->account);
strcpy(conn->init_dir,   old_conn->init_dir);
strcpy(conn->remote_dir, old_conn->remote_dir);
}

/* ------------------------
   | Save current options |
   ------------------------ */
void load_options(void)
{
long fh;

DEBUG();
fh = Fopen("drftp.cnf", FO_READ);
if (fh > -1)
	{
	long id, version;

	Fread((int)fh, sizeof(id), &id);
	if (id == 'DFTP')
		{
		Fread((int)fh, sizeof(version), &version);
		if (version < FTP_VERSION)
			Fread((int)fh, sizeof(ftp_opts) - 142, &ftp_opts);
		else
			Fread((int)fh, sizeof(ftp_opts), &ftp_opts);
		Fread((int)fh, sizeof(CONN_INF), &act_conndef);
	
		Fread((int)fh, sizeof(main_x), &main_x);
		Fread((int)fh, sizeof(main_y), &main_y);

		if (version < FTP_VERSION)
			main_y += 30;
	
		if (Fread((int)fh, sizeof(conn_cnt), &conn_cnt) != sizeof(conn_cnt))
			conn_cnt = 0;
			
		if (conn_cnt)
			{
			conn_defs = (CONN_INF *)malloc(conn_cnt * sizeof(CONN_INF));
			if (conn_defs)
				Fread((int)fh, sizeof(CONN_INF) * conn_cnt, conn_defs);
			else
				form_alert(1, mem_error);
			}
		}
	else
		{
		OLD_CONN_INF old_conn;
		
		Fseek(0, (int)fh, 0);
		Fread((int)fh, sizeof(ftp_opts) - 142, &ftp_opts);
			
		Fread((int)fh, sizeof(OLD_CONN_INF), &old_conn);
		convert_old(&old_conn, &act_conndef);
		
		Fread((int)fh, sizeof(main_x), &main_x);
		Fread((int)fh, sizeof(main_y), &main_y);

		main_y += 30;
	
		if (Fread((int)fh, sizeof(conn_cnt), &conn_cnt) != sizeof(conn_cnt))
			conn_cnt = 0;
			
		if (conn_cnt)
			{
			conn_defs = (CONN_INF *)malloc(conn_cnt * sizeof(CONN_INF));
			if (conn_defs)
				{
				int i;
				
				for (i = 0; i < conn_cnt; i++)
					{
					Fread((int)fh, sizeof(OLD_CONN_INF), &old_conn);
					convert_old(&old_conn, conn_defs + i);
					}
				}
			else
				form_alert(1, mem_error);
			}
		}
		
	Fclose((int)fh);
	}
copy_options(0);
}

/* ------------------------
   | Set/get radio button |
   ------------------------ */
static void opt_radio(OBJECT *tree, int *ids, int idnum, int *curr)
{
int i;

DEBUG();
if (!copy_mode)
	for (i = 0; i < idnum; i++)
		SetSelect(tree, ids[i], i == *curr);
else
	for (i = 0; i < idnum; i++)
		if (IsSelected(tree, ids[i]))
			{
			*curr = i;
			break;
			}
}

/* ------------------------
   | Set/get check button |
   ------------------------ */
static void opt_check(OBJECT *tree, int id, int *curr)
{
DEBUG();
if (!copy_mode)
	SetSelect(tree, id, *curr);
else
	*curr = IsSelected(tree, id);
}

/* --------------------------
   | Set/get tedinfo string |
   -------------------------- */
static void opt_string(OBJECT *tree, int id, char *curr)
{
DEBUG();
if (!copy_mode)
	strcpy(tree[id].ob_spec.tedinfo->te_ptext, curr);
else
	strcpy(curr, tree[id].ob_spec.tedinfo->te_ptext);
}

/* -------------------------
   | Set/get tedinfo value |
   ------------------------- */
static void opt_value(OBJECT *tree, int id, int *curr)
{
DEBUG();
if (!copy_mode)
	itoa(*curr, tree[id].ob_spec.tedinfo->te_ptext, 10);
else
	*curr = atoi(tree[id].ob_spec.tedinfo->te_ptext);
}

/* --------------------------------
   | Copy options from/to dialogs |
   -------------------------------- */
void copy_options(int direction)
{
OBJECT *x;
int i;

DEBUG();
copy_mode = direction;

rsrc_gaddr(R_TREE, DFTP, &x);
opt_radio(x, r_mode, 3, &ftp_opts.transfer_mode);

if (!direction)
	{
	make_colcell(option_dial, option_tabs[2] + SAME_COLOUR, ftp_opts.same);
	make_colcell(option_dial, option_tabs[2] + NEWER_COLOUR, ftp_opts.newer);
	make_colcell(option_dial, option_tabs[2] + OLDER_COLOUR, ftp_opts.older);
	make_colcell(option_dial, option_tabs[2] + NONEXIST_COLOUR, ftp_opts.nonexist);
	}
else
	{
	ftp_opts.same 		= (option_dial[option_tabs[2] + SAME_COLOUR].ob_state >> 8) & 0xFF;
	ftp_opts.newer 		= (option_dial[option_tabs[2] + NEWER_COLOUR].ob_state >> 8) & 0xFF;
	ftp_opts.older 		= (option_dial[option_tabs[2] + OLDER_COLOUR].ob_state >> 8) & 0xFF;
	ftp_opts.nonexist = (option_dial[option_tabs[2] + NONEXIST_COLOUR].ob_state >> 8) & 0xFF;
	}
/*
#define ICONS            10  /* BOXTEXT in Baum OPTAB1 */
#define ICONDROP         11  /* BUTTON in Baum OPTAB1 */
*/
opt_check(option_dial, 	option_tabs[0] + V_DELETE, 	&ftp_opts.v_delete);
opt_check(option_dial, 	option_tabs[0] + V_OVERWR, 	&ftp_opts.v_overwr);
opt_check(option_dial, 	option_tabs[0] + ICONCACHE, &ftp_opts.icon_cache);
opt_check(option_dial, 	option_tabs[0] + BUBBLEGEM, &ftp_opts.bubblegem);
opt_check(option_dial, 	option_tabs[0] + SV_WINDW, 	&ftp_opts.sv_window);
opt_value(option_dial, 	option_tabs[0] + TABSIZE, 	&ftp_opts.tab_size);

opt_check(option_dial, 	option_tabs[1] + EXT_COPY, 	&ftp_opts.ext_options);
opt_check(option_dial, 	option_tabs[1] + SHOWCONN, 	&ftp_opts.show_conn);
opt_check(option_dial, 	option_tabs[1] + PRESTIME, 	&ftp_opts.preserve);
opt_radio(option_dial, 	bps_mode, 2, 								&ftp_opts.bps_mode);
opt_value(option_dial, 	option_tabs[1] + IN_BUF, 		&ftp_opts.in_bufsize);
opt_value(option_dial, 	option_tabs[1] + OUT_BUF, 	&ftp_opts.out_bufsize);
opt_string(option_dial, option_tabs[1] + ANOEMAIL, 	ftp_opts.ano_email);

opt_radio(option_dial, 	l_sort, 4, &ftp_opts.l_sort);
opt_radio(option_dial, 	r_sort, 4, &ftp_opts.r_sort);
opt_check(option_dial, 	option_tabs[2] + V_LINKS, 	&ftp_opts.v_links);
opt_check(option_dial,  option_tabs[2] + COLOUR_FILES, &ftp_opts.colour_files);

opt_check(option_dial, 	option_tabs[3] + ENABECNV, 	&ftp_opts.do_extconv);

for (i = 0; i < 21; i++)
	{
	opt_string(option_dial, option_tabs[3] + cnve_ids[i], ftp_opts.remote_ext[i]);
	opt_string(option_dial, option_tabs[3] + cnvd_ids[i], ftp_opts.local_ext[i]);
	}
	
opt_check(option_dial, 	option_tabs[4] + ENABLOG, 	&ftp_opts.enable_log);
opt_check(option_dial, 	option_tabs[4] + LOGDATA, 	&ftp_opts.log_data);
opt_check(option_dial, 	option_tabs[4] + LOGCMDS, 	&ftp_opts.log_cmds);
opt_check(option_dial, 	option_tabs[4] + LOGERROR, 	&ftp_opts.log_error);
opt_string(option_dial, option_tabs[4] + LOGFILE, 	ftp_opts.logfile);

for (i = 0; i < 72; i++)
	opt_string(option_dial, option_tabs[5] + ascii_ids[i], 	ftp_opts.ascii[i]);
}

/* ------------------------------------
   | Copy connections from/to dialogs |
   ------------------------------------ */
void copy_connection(int direction)
{
DEBUG();
copy_mode = direction;

opt_string(connect_dial, connect_tabs[0] + CONNNAME, act_conndef.connection);
opt_string(connect_dial, connect_tabs[0] + SRV_NAME, act_conndef.host);
opt_string(connect_dial, connect_tabs[0] + USERNAME, act_conndef.user);
opt_string(connect_dial, connect_tabs[0] + PASSWORD, act_conndef.password);
opt_string(connect_dial, connect_tabs[0] + ACCNAME, 	act_conndef.account);
opt_check(connect_dial, 	connect_tabs[0] + ANOLOGIN, &act_conndef.anonymous);
opt_check(connect_dial, 	connect_tabs[0] + SAV_PWD, 	&act_conndef.save_password);

/*opt_value(connect_dial, connect_tabs[1] + MAXRETRY, 	&act_conndef.max_retries);*/
opt_value(connect_dial, connect_tabs[1] + TIMEOUT, 	&act_conndef.timeout_secs);
opt_value(connect_dial, connect_tabs[1] + PORTNUMB, 	&act_conndef.port_number);
opt_check(connect_dial, connect_tabs[1] + USEPASV, 	&act_conndef.passive_transfer);

opt_check(connect_dial, connect_tabs[2] + SHOWDOTS, 	&act_conndef.show_dots);
opt_radio(connect_dial, case_ids, 3, &act_conndef.case_mode);
opt_string(connect_dial, connect_tabs[2] + LOCALDIR, act_conndef.init_dir);
opt_string(connect_dial, connect_tabs[2] + REMOTDIR, act_conndef.remote_dir);
}

/* -------------------------------
   | Save connection information |
   ------------------------------- */
void save_connection(void)
{
int i;

DEBUG();
for (i = 0; i < conn_cnt; i++)
	if (!strcmp(conn_defs[i].connection, act_conndef.connection))
		{
		memcpy(conn_defs + i, &act_conndef, sizeof(CONN_INF));
		return;
		}

if (!conn_cnt)
	conn_defs = (CONN_INF *)malloc(sizeof(CONN_INF));
else
	conn_defs = (CONN_INF *)realloc(conn_defs, sizeof(CONN_INF) * (conn_cnt + 1));
 
for (i = 0; i < conn_cnt; i++)
	if (strcmp(conn_defs[i].connection, act_conndef.connection) > 0)
		{
		memmove(conn_defs + i + 1, conn_defs + i, (conn_cnt - i) * sizeof(CONN_INF));
		memcpy(conn_defs + i, &act_conndef, sizeof(CONN_INF));
		conn_cnt++;
		return;
		}

memcpy(conn_defs + conn_cnt, &act_conndef, sizeof(CONN_INF));
conn_cnt++;		
}

/* ---------------------------------
   | Delete connection information |
   --------------------------------- */
void delete_connection(void)
{
int i;

DEBUG();
for (i = 0; i < conn_cnt; i++)
	if (!strcmp(conn_defs[i].connection, act_conndef.connection))
		{
		if (i < conn_cnt - 1)
			memmove(conn_defs + i, conn_defs + i + 1,
							sizeof(CONN_INF) * (conn_cnt - i - 1));
		conn_cnt--;
		if (!conn_cnt)
			free(conn_defs);
		else
			conn_defs = (CONN_INF *)realloc(conn_defs, sizeof(CONN_INF) * conn_cnt);
		break;
		}
}

/* -----------------------------
   | Get a specific connection |
   ----------------------------- */
void get_connection(char *cname)
{
int i;

DEBUG();
for (i = 0; i < conn_cnt; i++)
	if (!strcmp(conn_defs[i].connection, cname))
		{
		memcpy(&act_conndef, conn_defs + i, sizeof(CONN_INF));
		break;
		}
}
