/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"
#include "..\include\color.h"

extern int case_ids[], bps_mode[], l_sort[], r_sort[];

static int alertid[] = {NODRIVER, YOFFLINE, UNK_HOST, UNK_SERV,
												NO_CONN, LOGINERR, NOGET, NOMEM,
												NORESTAR, CREATERR, NOLOGFIL,
												UNABLOG, TOMANYWI, UNABOPEN,
												ALLOCERR, READERR, SCRAPERR,
												NOTPALET, 
												};

static char **alerts[]={&nodriver_msg, &offline_msg, 
												&unknown_host, &unknown_service,
												&connect_unable, &login_error,
												&get_error, &mem_error,
												&restart_error, &create_error,
												&no_logfile, &cantopen_log,
												&tomany_windows, &open_error,
												&alloc_error, &read_error, &scrap_error,
												&notpalet,
												};

/* -----------------------
   | Initialize resource |
   ----------------------- */
int init_rsc(void)
{
static int conntabs[] = {CONNOPT1, CONNOPT2, CONNOPT3};
static int optabs[] = {OPTAB1, OPTAB2, OPTAB3, OPTAB4, OPTAB5, OPTAB6};
OBJECT *tree, *tab_x;
int i;
GRECT fr;

DEBUG();
if (!rsrc_load("DRACFTP.RSC"))
	{
	form_alert(1,"[3][Resource-file|'DRACFTP.RSC' missing.][Oops!]");
	return FALSE;
	}

connect_dial = NULL;
rsrc_gaddr(R_TREE, CONNECT, &tree);
for (i = 0; i < 3; i++)
	{
	rsrc_gaddr(R_TREE, conntabs[i],	&tab_x);
	if (!i)
		connect_tabs[i] = CombineDialogs(&connect_dial, tree, tab_x);
	else
		connect_tabs[i] = CombineDialogs(&connect_dial, connect_dial, tab_x);
	}
for (i = 0; i < 3; i++)
	case_ids[i] += connect_tabs[2];

option_dial = NULL;
rsrc_gaddr(R_TREE, OPTIONS, &tree);
for (i = 0; i < 6; i++)
	{
	rsrc_gaddr(R_TREE, optabs[i],	&tab_x);
	if (!i)
		option_tabs[i] = CombineDialogs(&option_dial, tree, tab_x);
	else
		option_tabs[i] = CombineDialogs(&option_dial, option_dial, tab_x);
	}
for (i = 0; i < 2; i++)		bps_mode[i] += option_tabs[1];
for (i = 0; i < 4; i++)		l_sort[i] += option_tabs[2];
for (i = 0; i < 4; i++)		r_sort[i] += option_tabs[2];
	
rsrc_gaddr(R_STRING, ERRWRITE, &error_write);
rsrc_gaddr(R_TREE, CPYINFO, &cpyinfo);

rsrc_gaddr(R_TREE, STRINGS, &tree);
recvstring 	= tree[STR_01].ob_spec.free_string;
sendstring 	= tree[STR_02].ob_spec.free_string;
byte_info 	= tree[STR_04].ob_spec.free_string;
bps_info 		= tree[STR_05].ob_spec.free_string;
byt_info 		= tree[STR_06].ob_spec.free_string;
std_file		= tree[STDFILE].ob_spec.free_string;
dir_type		= tree[DIRTYPE].ob_spec.free_string;
link_type		= tree[LINKTYPE].ob_spec.free_string;
aborted			= tree[ABORTED].ob_spec.free_string;
done				= tree[DONE].ob_spec.free_string;
readdir			= tree[READDIR].ob_spec.free_string;
connecting  = tree[CONNECTI].ob_spec.free_string;
userauth		= tree[USERAUTH].ob_spec.free_string;
connected	  = tree[CONNECTD].ob_spec.free_string;
closeconnection = tree[DISCONNE].ob_spec.free_string;
unknown			= tree[UNKNOWN].ob_spec.free_string;
languagecode= tree[LANGUAGECODE].ob_spec.free_string;
chdir_label	= tree[CHDIR_LABEL].ob_spec.free_string;
site_label	= tree[SITE_LABEL].ob_spec.free_string;
quote_label	= tree[QUOTE_LABEL].ob_spec.free_string;
logfilelabel	= tree[LOGFILELABEL].ob_spec.free_string;

/* Alerts */
for (i = 0; i < sizeof(alertid) / sizeof(int); i++)
	rsrc_gaddr(R_STRING, alertid[i],  alerts[i]);

rsrc_gaddr(R_TREE, ICONIFD, &iconified);
form_center(iconified, &fr);
rsrc_gaddr(R_TREE, VICONIFD, &v_iconified);
form_center(v_iconified, &fr);

rsrc_gaddr(R_TREE, CSELPOP, &tree);
create_cpopup( tree);

#ifdef EVAL
	rsrc_gaddr(R_TREE, NOTDEMO, &notdemo);
#endif

return TRUE;
}
