/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

static int only_newer, recursive, confirm;

/* -----------------------
   | Send selected files |
   ----------------------- */
void send_files(int callflg)
{
int i, j, flg;
struct ftp_lsblk *dirfiles = NULL;
int dircnt = 0;
char *ftp_dirbackup, *local_dirbackup;

DEBUG();
#ifdef EVAL
	if (transferred)
		{
		OBJECT *onlyone;
		
		rsrc_gaddr(R_TREE, ONLYONE, &onlyone);
		open_kinddialog(onlyone, NULL, iconified, SMALLER|NAME|MOVER|CLOSER);
		do_dialog(onlyone, 0, app_callback);
		close_dialog(onlyone, TRUE);

		return;
		}
#endif

if (callflg && ftp_opts.ext_options)
	{
	int has_dir = FALSE, cnt = 0;
	
	for (i = 0; i < local_filecnt; i++)
		if (localfiles[i].sel)
			{
			cnt++;
			if (localfiles[i].dir) has_dir = TRUE;
			}
	
	if (has_dir || cnt > 1)
		{
		static int copy_help[] = {COPY_RECURSIVE, ONLY_NEW,
															CONFIRM_OVERW, COPT_CANCEL,
															COPT_OK, 0};
		OBJECT *copyoptions;
		int obj;
		
		rsrc_gaddr(R_TREE, COPYOPTIONS, &copyoptions);
		if (has_dir)
			{
			SetSelect(copyoptions, COPY_RECURSIVE, ftp_opts.recursive);
			}
		else
			{
			DeselectObj(copyoptions, COPY_RECURSIVE);
			}

		SetSelect(copyoptions, ONLY_NEW, ftp_opts.only_newer);
		SetSelect(copyoptions, CONFIRM_OVERW, ftp_opts.confirm_overwr);

		open_kinddialog(copyoptions, copyoptions[COPYOPT_TITLE].ob_spec.free_string, iconified, SMALLER|NAME|MOVER|CLOSER);
		dlgset_help_info(copyoptions, helpfile, copy_help, "COPY_");
		obj = do_dialog(copyoptions, 0, app_callback);
		close_dialog(copyoptions, TRUE);
		
		if (obj != COPT_OK)
			return;
		
		if (has_dir)
			recursive = ftp_opts.recursive = IsSelected(copyoptions, COPY_RECURSIVE);
		else
			recursive = FALSE;
		only_newer = ftp_opts.only_newer = IsSelected(copyoptions, ONLY_NEW);
		confirm = ftp_opts.confirm_overwr = IsSelected(copyoptions, CONFIRM_OVERW);
		}
	else
		{
		recursive = FALSE;
		only_newer = FALSE;
		confirm = ftp_opts.confirm_overwr;
		}
	}
	
if (IsSelected(maindial, BINMODE))
	ftp_binary(ftp_socket, 1);
else
	if (IsSelected(maindial, ASCIMODE))
		ftp_binary(ftp_socket, 0);
	
if (ftp_socket)
	{
	flg = FALSE;
	for (i = 0; i < local_filecnt; i++)
		if (localfiles[i].sel)
			{
			if (!localfiles[i].dir)
				{
				if (ftp_opts.only_newer || localfiles[i].newer)
				  {
					switch (send_file(localfiles[i].name, localfiles[i].size))
						{
						case 0:
							break;
					
						case 1:		
							flg = TRUE;
							break;
						
						case -2:
							flg |= 0x8000;
							break;
						}
					if (flg & 0x8000) break;
					}
				}
			else
				{
				if (ftp_mkdir(ftp_socket, localfiles[i].name))
					flg = TRUE;

				if (recursive)
					{
					if (!dircnt)
						dirfiles = (struct ftp_lsblk *)malloc(sizeof(struct ftp_lsblk));
					else
						dirfiles = (struct ftp_lsblk *)realloc(dirfiles, sizeof(struct ftp_lsblk) * (dircnt + 1));
				
					memcpy(dirfiles + dircnt, localfiles + i, sizeof(struct ftp_lsblk));
					dircnt++;
					}
				}
			}

	if (recursive && dircnt && !(flg & 0x8000))
		{
		ftp_dirbackup = (char *)malloc(256);
		local_dirbackup = (char *)malloc(256);
		strcpy(ftp_dirbackup, ftp_remote);
		strcpy(local_dirbackup, local_dir);
		
		for (i = 0; i < dircnt; i++)
			{
			if (ftp_setcwd(ftp_socket, dirfiles[i].name))
				{
				ftp_getcwd(ftp_socket, ftp_remote);
				ReadDirectory();
					
				SetInfo(done);
				
				strcpy(local_dir, local_dirbackup);
				if (strchr(local_dir, 0)[-1] != '\\')
					strcat(local_dir, "\\");
				strcat(local_dir, dirfiles[i].name);
				GetLocaldir();
				
				for (j = 0; j < local_filecnt; j++)
					if (strcmp(localfiles[j].name, ".") && strcmp(localfiles[j].name, ".."))
						localfiles[j].sel ^= 1;
				send_files(FALSE);
				}

			if (ftp_setcwd(ftp_socket, ftp_dirbackup))
				flg = TRUE;
			}

		if (strcmp(local_dir, local_dirbackup))
			{
			strcpy(local_dir, local_dirbackup);
			GetLocaldir();
			}
				
		free(ftp_dirbackup);
		free(local_dirbackup);
		}
		
	if (dircnt)
		free(dirfiles);
		
	if (flg && callflg)
		{
		ReadDirectory();
		SetInfo(done);
		}
	}
	
#ifdef EVAL
	transferred = TRUE;
#endif
}

/* -------------------
   | Send local file |
   ------------------- */
int send_file(char *localfile, long totalsize)
{
char fb[256], remotefile[256], *sndbuf;
long a, sent;
int bufsize, restart = 0, idx, i, ret;

DEBUG();
if (IsSelected(maindial, AUTOMODE))
	if (CheckTransfermode(localfile) == 1)
		ftp_binary(ftp_socket, 1);
	else
		ftp_binary(ftp_socket, 0);

strcpy(remotefile, localfile);
switch(act_conndef.case_mode)
	{
	case 1:
		strupr(remotefile);
		break;
		
	case 2:
		strlwr(remotefile);
		break;
	}
replaceLocal(remotefile);

sent = 0;
if (confirm)
	{
	idx = -1;
	for (i = 0; i < ftp_filecnt; i++)
		if (!strcmp(remotefile, ftpfiles[i].name))
			{
			idx = i;
			break;
			}
	
	if (idx > -1)
		{
		static int trf_help[] = {TRF_CONT, TRFCANCL, TRFOVER, 0};
		OBJECT *ovr;
		int obj;
		
		rsrc_gaddr(R_TREE, OVERWR, &ovr);
	
	#ifdef EVAL
		DisableObj(ovr, TRF_CONT);
	#endif
		
		open_kinddialog(ovr, ovr[OVERWR_TITLE].ob_spec.free_string, iconified, SMALLER|NAME|MOVER|CLOSER);
		dlgset_help_info(ovr, helpfile, trf_help, "TRF_");
		obj = do_dialog(ovr, 0, app_callback);
		close_dialog(ovr, TRUE);
		
		switch(obj)
			{
			case TRFOVER:
				break;
			
			case TRF_CONT:
				sent = ftpfiles[idx].size;
				restart = 1;
				break;
			
			case 0x2000|WM_CLOSED:
			case TRFCANCL:
				return FALSE;
			}
		}
	}
		
if (restart)
	{
	ret = ftp_store(ftp_socket, remotefile, sent);
	if (ret == -2)
		{
		if (form_alert(1, restart_error) == 1)
			{
			restart = 0;
			sent = 0;
			ret = ftp_store(ftp_socket, remotefile, 0L);
			}
		else
			return FALSE;
		}
	}
else
	ret = ftp_store(ftp_socket, remotefile, 0L);
	
if (ret)
	{
	SetInfo(sendstring);

	strcpy(fb, local_dir);
	strcat(fb, "\\");
	strcat(fb, localfile);
	
	sndbuf = (char *)malloc(ftp_opts.out_bufsize);

	transmission_start = clock();
 	open_cpyinfo(1, remotefile);
		
	a = Fopen(fb, FO_READ);
	Fseek(sent, (int)a, SEEK_SET);
	
	while((bufsize = (int)Fread((int)a, ftp_opts.out_bufsize, sndbuf)) > 0)
		{
#ifdef EVAL
		if (((sent + bufsize) >> 10) > 30)
			{
			OBJECT *only30;

			close_cpyinfo();
			Fclose((int)a);
			free(sndbuf);
		
			rsrc_gaddr(R_TREE, ONLY30KB, &only30);
			open_kinddialog(only30, NULL, iconified, SMALLER|NAME|MOVER|CLOSER);
			do_dialog(only30, 0, app_callback);
			close_dialog();
			
			if (ftp_putblock(ftp_socket, 0, NULL) == -1)
				{}
			SetInfo(aborted);
			return TRUE;
			}
#endif

		check_events();
		if (!is_cpyinfoOpen())
			{
			close_cpyinfo();
			Fclose((int)a);
			free(sndbuf);

			if (ftp_putblock(ftp_socket, 0, NULL) == -1)
				{}
			SetInfo(aborted);
			return -2;
			}

		if (ftp_putblock(ftp_socket, bufsize, sndbuf) != bufsize)
			break;
			
		sent += bufsize;
		
		set_cpyinfo(sent, totalsize);
		}

	if (ftp_putblock(ftp_socket, 0, NULL) == -1)
		{}

	close_cpyinfo();

	free(sndbuf);
	Fclose((int)a);
	SetInfo(done);

	return TRUE;
	}
return FALSE;
}
