/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-07-15
 * 
 */
 
#include "extern.h"

typedef struct viewinfo
											{
											int  w_handle;
											long size;
											int  binmode, iconified;
											int  vsldpos, vsldsize;
											int  hsldpos, hsldsize;
											long top, lines, asc_lines;
											int  left, width, asc_width;
											unsigned char *ptr;
											long mark_start, mark_end;
											char title[61];
											}VIEWINFO;

VIEWINFO viewinfos[20];
int viewinfo_cnt = 0;

/* Local prototype */
void calc_viewlines(VIEWINFO *vinf, int ww);
void calc_asciilines(VIEWINFO *vinf);
long get_mark(void);

extern char *logmem;

/* ---------------------
   | View remote files |
   --------------------- */
void view_locals(void)
{
int i, binmode;
char fb[256];

DEBUG();
if (IsSelected(maindial, BINMODE))	
	binmode = 1;
else
	if (IsSelected(maindial, ASCIMODE))
		binmode = 0;

for (i = 0; i < local_filecnt; i++)
	if (localfiles[i].sel)
		{
		if (IsSelected(maindial, AUTOMODE))
			if (CheckTransfermode(localfiles[i].name) == 1)
				binmode = 1;
			else
				binmode = 0;
				
		strcpy(fb, local_dir);
		strcat(fb, "\\");
		strcat(fb, localfiles[i].name);
		
		view_file(fb, binmode, localfiles[i].name);
		}
}
											
/* ---------------------
   | View remote files |
   --------------------- */
void view_remotes(void)
{
int i, binmode;
char *tmp;

DEBUG();
if (IsSelected(maindial, BINMODE))	
	{
	ftp_binary(ftp_socket, 1);
	binmode = 1;
	}
else
	if (IsSelected(maindial, ASCIMODE))
		{
		ftp_binary(ftp_socket, 0);
		binmode = 0;
		}

if (ftp_socket)
	{
	for (i = 0; i < ftp_filecnt; i++)
		if (ftpfiles[i].sel)
			{
			tmp = tmpnam(NULL);
			if (receive_file(ftpfiles + i, tmp))
				{
				if (IsSelected(maindial, AUTOMODE))
					if (CheckTransfermode(ftpfiles[i].name) == 1)
						binmode = 1;
					else
						binmode = 0;
				
				view_file(tmp, binmode, ftpfiles[i].name);
#ifndef __AHCC__
				unlink(tmp);
#else
				Fdelete(tmp);
#endif		
			}
		}
	}
}

/* -----------------------------------
   | Calculate number of ASCII lines |
   ----------------------------------- */
void calc_asciilines(VIEWINFO *vinf)
{
unsigned char *ptr;
long mem_len;
int cnt;

DEBUG();
vinf->asc_lines = 0L;
vinf->asc_width = 0;
ptr = vinf->ptr;
mem_len = vinf->size;
while(mem_len)
	{
	vinf->asc_lines++;
	cnt = 0;
	while(mem_len && *ptr != '\n' && *ptr != '\r')
		{
		if (*ptr == '\t')
			do
				{
				cnt++;
				}while(cnt % ftp_opts.tab_size);
		else
			cnt++;
		ptr++;
		mem_len--;
		}
		
	if (cnt > vinf->asc_width) vinf->asc_width = cnt;
	
	if (mem_len)
		{
		if (mem_len > 1 && *ptr != ptr[1] && (ptr[1] == '\r' || ptr[1] == '\n'))
			{
			ptr++;
			mem_len--;
			}
		
		ptr++;
		mem_len--;
		}
	}
}

/* --------------------
   | Mail view slider |
   -------------------- */
void set_viewslider(VIEWINFO *vinf)
{
int wx, wy, ww, wh;
int size, pos;

DEBUG();
wind_get(vinf->w_handle, WF_WORKXYWH, &wx, &wy, &ww, &wh);
	
if (ww >= vinf->width * tx_width)
	{
	size = 1000;
	pos = 0;
	}
else
	{
	pos = (int)(vinf->left * 1000L / (vinf->width - ww / tx_width));
	size = (int)(ww / tx_width * 1000L / vinf->width);
	}
if (vinf->hsldsize != size)	wind_set_int(vinf->w_handle, WF_HSLSIZE, size);
if (vinf->hsldpos  != pos)	wind_set_int(vinf->w_handle, WF_HSLIDE, pos);
vinf->hsldsize = size;
vinf->hsldpos = pos;

if (wh / tx_height >= vinf->lines)
	{
	size = 1000;
	pos = 0;
	}
else
	{
	pos = (int)(vinf->top * 1000L / (vinf->lines - wh / tx_height));
	size = (int)((wh / tx_height) * 1000L / vinf->lines);
	}
if (vinf->vsldsize != size)	wind_set_int(vinf->w_handle, WF_VSLSIZE, size);
if (vinf->vsldpos  != pos)	wind_set_int(vinf->w_handle, WF_VSLIDE, pos);
vinf->vsldsize = size;
vinf->vsldpos = pos;
}

/* -------------
   | View file |
   ------------- */
void view_file(char *file, int binmode, char *title)
{
int wx, wy, ww, wh;
GRECT wrec, mrec;
int w_handle;
long fh, size;

DEBUG();
if (viewinfo_cnt == 20)
	{
	form_alert(1, tomany_windows);
	return;
	}

if (file)
	{
	fh = Fopen(file, FO_READ);
	if (fh < 0)
		{
		form_alert(1, open_error);
		return;
		}
	
	size = Fseek(0L, (int)fh, 2);
	Fseek(0L, (int)fh, 0);
	
	if (size)
		{
		viewinfos[viewinfo_cnt].ptr = (unsigned char *)malloc(size);
		if (!viewinfos[viewinfo_cnt].ptr)
			{
			Fclose((int)fh);
			form_alert(1, alloc_error);
			return;
			}
		}
	else
		viewinfos[viewinfo_cnt].ptr = NULL;
	viewinfos[viewinfo_cnt].size = size;
	
	if (Fread((int)fh, size, viewinfos[viewinfo_cnt].ptr) != size)
		{
		Fclose((int)fh);
		form_alert(1, read_error);
		free(viewinfos[viewinfo_cnt].ptr);
		return;
		}
	Fclose((int)fh);
	}
else
	if (logmem)
		{
		viewinfos[viewinfo_cnt].ptr = (unsigned char *)logmem;
		viewinfos[viewinfo_cnt].size = strlen(logmem);
		logmem = NULL;
		}
	else
		return;
		
wind_get_grect(ROOT, WF_WORKXYWH, &wrec);

viewinfos[viewinfo_cnt].w_handle = 
w_handle = wind_create(SMALLER|NAME|CLOSER|FULLER|MOVER|SIZER|UPARROW|DNARROW|
											 VSLIDE|LFARROW|RTARROW|HSLIDE,
											 &wrec);
if (w_handle < 0)
	{
	free(viewinfos[viewinfo_cnt].ptr);
	form_alert(1, tomany_windows);
	return;
	}
									 
strncpy(viewinfos[viewinfo_cnt].title, title, 60);
viewinfos[viewinfo_cnt].title[60] = 0;

wind_set_string(w_handle, WF_NAME, viewinfos[viewinfo_cnt].title);

calc_asciilines(viewinfos + viewinfo_cnt);

if (!binmode)
	{
	mrec.g_x = mrec.g_y = 0;
	mrec.g_w = tx_width * viewinfos[viewinfo_cnt].asc_width;
	mrec.g_h = 0;
	
	wind_calc(WC_BORDER, SMALLER|NAME|CLOSER|FULLER|MOVER|SIZER|UPARROW|DNARROW|
											 VSLIDE|LFARROW|RTARROW|HSLIDE, &mrec, &mrec);
	if (mrec.g_h < 75) mrec.g_h = 75;
	if (mrec.g_w < 100) mrec.g_w = 100;
	
	if (wrec.g_h > mrec.g_h + tx_height * viewinfos[viewinfo_cnt].asc_lines)
		{
		int n = mrec.g_h + tx_height * viewinfos[viewinfo_cnt].asc_lines;
		wrec.g_y += (wrec.g_h - n) >> 1;
		wrec.g_h = n;
		}
		
	if (wrec.g_w > mrec.g_w)
		{
		wrec.g_x += (wrec.g_w - mrec.g_w) >> 1;
		wrec.g_w = mrec.g_w;
		}
	}
		
wind_open(w_handle, &wrec);

viewinfos[viewinfo_cnt].mark_start = 
viewinfos[viewinfo_cnt].mark_end = -1L;

viewinfos[viewinfo_cnt].top =
viewinfos[viewinfo_cnt].lines = 0L;

viewinfos[viewinfo_cnt].width =
viewinfos[viewinfo_cnt].left = 0;
viewinfos[viewinfo_cnt].iconified = FALSE;
viewinfos[viewinfo_cnt].binmode = binmode;

viewinfos[viewinfo_cnt].hsldsize =
viewinfos[viewinfo_cnt].hsldpos =
viewinfos[viewinfo_cnt].vsldsize =
viewinfos[viewinfo_cnt].vsldpos = -1;

wind_get(w_handle, WF_WORKXYWH, &wx, &wy, &ww, &wh);

calc_viewlines(viewinfos + viewinfo_cnt, ww);
set_viewslider(viewinfos + viewinfo_cnt);

viewinfo_cnt++;
}

/* ------------------------
   | Is it a view window? |
   ------------------------ */
int is_view(int w_handle)
{
int i;

DEBUG();
for (i = 0; i < viewinfo_cnt; i++)
	if (viewinfos[i].w_handle == w_handle)
		return TRUE;

return FALSE;
}

/* ------------------------
   | Is it a view window? |
   ------------------------ */
VIEWINFO *get_viewinfo(int w_handle)
{
int i;

DEBUG();
for (i = 0; i < viewinfo_cnt; i++)
	if (viewinfos[i].w_handle == w_handle)
		return viewinfos + i;

return NULL;
}

/* --------------
   | Close view |
   -------------- */
void close_view(VIEWINFO *vinf)
{
DEBUG();
wind_close(vinf->w_handle);
wind_delete(vinf->w_handle);

if (vinf->ptr) free(vinf->ptr);

if (vinf < viewinfos + viewinfo_cnt - 1)
	memmove(vinf, vinf + 1, sizeof(VIEWINFO) * 
													(viewinfo_cnt - (vinf - viewinfos) - 1));
viewinfo_cnt--;
}

/* ---------------
   | View binary |
   --------------- */
void view_bin(int wx, int wy, int ww, int wh, VIEWINFO *vinf)
{
int bytes_per_line, lines, i, j, line_size, offset;
unsigned char *ptr, *end, c;
char *buf;
static char hexchar[] = "0123456789ABCDEF";

DEBUG();
bytes_per_line = 0;
while ((bytes_per_line * 4 + 7 + 2) * tx_width < ww)
	bytes_per_line++;
bytes_per_line--;

if (bytes_per_line <= 0) return;

line_size = bytes_per_line * 4 + 7 + 2;
buf = (char *)malloc(line_size + 1);

lines = (wh + tx_height - 1) / tx_height;

ptr = vinf->ptr + vinf->top * bytes_per_line;
end = vinf->ptr + vinf->size;

buf[line_size] = 0;
for (i = 0; i < lines && ptr < end; i++)
	{
	sprintf(buf, "%06lx", ptr - vinf->ptr);
	
	memset(buf + 6, ' ', line_size - 6);
	
	for (j = 0; j < bytes_per_line && ptr + j < end; j++)
		{
		c = ptr[j];
		buf[7 + j * 3] 			= hexchar[c >> 4];
		buf[7 + j * 3 + 1] 	= hexchar[c & 0xF];
		
		if (!isprint(c)) c = '.';
		
		buf[bytes_per_line * 3 + 9 + j] = c;
		}

	if (vinf->mark_start != -1)
		{
		if (ptr + bytes_per_line < vinf->ptr + vinf->mark_start ||
				ptr >= vinf->ptr + vinf->mark_end)	/* outside of marker */
			{
			vswr_mode(vdi_handle, MD_REPLACE);
			v_gtext(vdi_handle, wx, wy + i * tx_height, buf);
			}
		else
			{
			if (ptr >= vinf->ptr + vinf->mark_start &&
					ptr + bytes_per_line <= vinf->ptr + vinf->mark_end)	/* complete marked */
				{
				buf[6] = 0;
				v_gtext(vdi_handle, wx, wy + i * tx_height, buf);
				v_gtext(vdi_handle, wx + 6 * tx_width, wy + i * tx_height, " ");

				vswr_mode(vdi_handle, MD_ERASE);
				buf[bytes_per_line * 3 + 6] = 0;
				v_gtext(vdi_handle, wx + 7 * tx_width, wy + i * tx_height, buf + 7);

				v_gtext(vdi_handle, wx + (bytes_per_line * 3 + 9) * tx_width, 
														wy + i * tx_height, 
														buf + (bytes_per_line * 3 + 9));
				vswr_mode(vdi_handle, MD_REPLACE);
				}
			else
				{					/* partly marked */
				vswr_mode(vdi_handle, MD_ERASE);
				v_gtext(vdi_handle, wx + 7 * tx_width, wy + i * tx_height, buf + 7);
				vswr_mode(vdi_handle, MD_REPLACE);
				
				buf[6] = 0;
				v_gtext(vdi_handle, wx, wy + i * tx_height, buf);
				v_gtext(vdi_handle, wx + (bytes_per_line * 3 + 6) * tx_width, 
														wy + i * tx_height, "   ");
				
				if (ptr < vinf->ptr + vinf->mark_start)
					{
					buf[((vinf->ptr + vinf->mark_start) - ptr) * 3 + 7] = 0;
					v_gtext(vdi_handle, wx + 7 * tx_width, wy + i * tx_height, buf + 7);
					
					offset = bytes_per_line * 3 + 9 + (int)((vinf->ptr + vinf->mark_start) - ptr);
					buf[offset] = 0;
					v_gtext(vdi_handle, wx + (bytes_per_line * 3 + 9) * tx_width, 
															wy + i * tx_height, 
															buf + bytes_per_line * 3 + 9);
					}
	
				if (ptr + bytes_per_line > vinf->ptr + vinf->mark_end)
					{
					offset = (int)(vinf->ptr + vinf->mark_end - ptr);
					buf[bytes_per_line * 3 + 6] = 0;
					v_gtext(vdi_handle, wx + (6 + offset * 3) * tx_width, 
									wy + i * tx_height, 
									buf + offset * 3 + 6);

					v_gtext(vdi_handle, 
									wx + (bytes_per_line * 3 + 9 + offset) * tx_width, 
									wy + i * tx_height, 
									buf + bytes_per_line * 3 + 9 + offset);
					}
				}
			}
		}
	else			
		v_gtext(vdi_handle, wx, wy + i * tx_height, buf);
		
	ptr += bytes_per_line;
	}

free(buf);
}

/* -------------------
   | View ASCII text |
   ------------------- */
void view_ascii(int wx, int wy, int ww, int wh, VIEWINFO *vinf)
{
char *buf, *lput;
unsigned char *mem, *memx, sc, ec;
long mem_len;
long pg;
int hg, i, mw, cpos, xpos, soff, eoff;
int xy[4];

DEBUG();
vsl_color(vdi_handle, 1);
vsl_type(vdi_handle, 1);
vsl_width(vdi_handle, 1);

mem = vinf->ptr;
mem_len = vinf->size;
pg = vinf->top;
while(pg--)
	{
	while(mem_len && *mem != '\n' && *mem != '\r')
		{
		mem++;
		mem_len--;
		}
	
	if (mem_len)
		{
		if (mem_len > 1 && *mem != mem[1] && (mem[1] == '\r' || mem[1] == '\n'))
			{
			mem++;
			mem_len--;
			}
		
		mem++;
		mem_len--;
		}
	}
	
hg = (wh + tx_height - 1) / tx_height;
mw = (ww + tx_width - 1) / tx_width;
buf = (char *)malloc(mw + 1);
for (i = 0; i < hg && mem_len; i++)
	{
	xpos = cpos = 0;
	lput = buf;
	memx = mem;
	soff = eoff = -1;
	
	while(mem_len && *mem != '\n' && *mem != 12 && *mem != '\r')
		{
		if (mem == vinf->ptr + vinf->mark_start) soff = cpos;
		if (mem == vinf->ptr + vinf->mark_end) 	eoff = cpos;
		
		if (*mem != '\t')
			{
			if (xpos >= vinf->left && cpos < mw)
				{
				*lput++ = *mem;
				cpos++;
				}
			mem++;
			xpos++;
			}
		else
			{
			do
				{
				if (xpos >= vinf->left && cpos < mw)
					{
					*lput++ = ' ';
					cpos++;
					}
				xpos++;
				}while(xpos % ftp_opts.tab_size);
			mem++;
			}
			
		mem_len--;
		}

	if (cpos)
		{
		buf[cpos] = 0;
		
		if (vinf->mark_start != -1 &&
				memx < vinf->ptr + vinf->mark_end &&
				mem > vinf->ptr + vinf->mark_start)
			{
			if (soff == -1 && eoff == -1)
				{
				vswr_mode(vdi_handle, MD_ERASE);
				v_gtext(vdi_handle, wx, wy, buf);
				vswr_mode(vdi_handle, MD_REPLACE);
				}
			else
				{
				if (soff > -1)
					{
					sc = buf[soff];
					buf[soff] = 0;
					v_gtext(vdi_handle, wx, wy, buf);
					buf[soff] = sc;
					}
				else
					soff = 0;
				
				if (eoff > -1)
					{
					ec = buf[eoff];
					buf[eoff] = 0;
					}
					
				vswr_mode(vdi_handle, MD_ERASE);
				v_gtext(vdi_handle, wx + soff * tx_width, wy, buf + soff);
				vswr_mode(vdi_handle, MD_REPLACE);
			
				if (eoff > -1)
					{
					buf[eoff] = ec;
					v_gtext(vdi_handle, wx + eoff * tx_width, wy, buf + eoff);
					}
				}
			}		
		else
			v_gtext(vdi_handle, wx, wy, buf);
		}

	if (*mem != 12)
		{
		if (mem_len)
			{
			if (mem_len > 1 && *mem != mem[1] && (mem[1] == '\r' || mem[1] == '\n')) 
				{
				mem++;
				mem_len--;
				}
				
			mem++;
			mem_len--;
			}
		}
	else
		{
		mem++;
		mem_len--;
		xy[0] = wx;
		xy[3] = xy[1] = wy + tx_height;
		xy[2] = wx + ww - 1;
		v_pline(vdi_handle, 2, xy);
		}
		
	wy += tx_height;
	}
free(buf);
}

/* ----------------------
   | Redraw object tree |
   ---------------------- */
void redraw_view(VIEWINFO *vinf, int rx, int ry, int rw, int rh)
{
GRECT t1, t2;
int wx, wy, ww, wh;
int clip[4];

DEBUG();
t2.g_x = rx;
t2.g_y = ry;
t2.g_w = rw;
t2.g_h = rh;

wind_update(BEG_UPDATE);
graf_mouse(M_OFF, 0L);

wind_get(vinf->w_handle, WF_WORKXYWH, &wx, &wy, &ww, &wh);
v_iconified[0].ob_x = wx;
v_iconified[0].ob_y = wy;
v_iconified[0].ob_width = ww;
v_iconified[0].ob_height = wh;

v_iconified[1].ob_x = (ww - v_iconified[1].ob_width) >> 1;
v_iconified[1].ob_y = (wh - v_iconified[1].ob_height) >> 1;

wind_get_grect(vinf->w_handle, WF_FIRSTXYWH, &t1);
while(t1.g_w && t1.g_h)
	{
	if (rc_intersect(&t2, &t1))
		if (vinf->iconified)
			objc_draw(v_iconified, 0, MAX_DEPTH, &t1);
		else
			{
			clip[0] = t1.g_x;
			clip[1] = t1.g_y;
			clip[2] = t1.g_x + t1.g_w - 1;
			clip[3] = t1.g_y + t1.g_h - 1;
			vs_clip(vdi_handle, 1, clip);
			
			vsf_interior(vdi_handle, FIS_HOLLOW);
			vr_recfl(vdi_handle, clip);
			
			if (vinf->binmode)
				view_bin(wx, wy, ww, wh, vinf);
			else
				view_ascii(wx, wy, ww, wh, vinf);
				
			vs_clip(vdi_handle, 0, clip);
			}

	wind_get_grect(vinf->w_handle, WF_NEXTXYWH, &t1);
	}

graf_mouse(M_ON, 0L);
wind_update(END_UPDATE);
}

/* ------------------------
   | Calculate view lines |
   ------------------------ */
void calc_viewlines(VIEWINFO *vinf, int ww)
{
int bytes_per_line;

DEBUG();
if (vinf->binmode)
	{
	bytes_per_line = 0;
	while ((bytes_per_line * 4 + 7 + 2) * tx_width < ww)
		bytes_per_line++;
	bytes_per_line--;
	
	vinf->lines = (vinf->size + bytes_per_line - 1) / bytes_per_line;
	vinf->width = (bytes_per_line * 4 + 7 + 2);
	}
else
	{
	vinf->lines = vinf->asc_lines;
	vinf->width = vinf->asc_width;
	}
}

/* -------------------------------
   | Process view window message |
   ------------------------------- */
void view_msg(int *msg)
{
VIEWINFO *vinf;
int wx, wy, ww, wh;
int cx, cy, cw, ch;

DEBUG();
vinf = get_viewinfo(msg[3]);
if (!vinf)
	return;
		
wind_get(msg[3], WF_WORKXYWH, &wx, &wy, &ww, &wh);

switch(msg[0])
	{
	case WM_CLOSED:
		close_view(vinf);
		break;
		
	case WM_REDRAW:
		redraw_view(vinf, msg[4], msg[5], msg[6], msg[7]);
		break;

	case WM_TOPPED:
	case WM_NEWTOP:
		wind_set_int(msg[3], WF_TOP, 0);
		break;

	case WM_ICONIFY:
	case WM_ALLICONIFY:
		vinf->iconified = TRUE;
		wind_set(msg[3], WF_ICONIFY, msg[4], msg[5], msg[6], msg[7]);
		break;

	case WM_UNICONIFY:
		vinf->iconified = FALSE;
		wind_set(msg[3], WF_UNICONIFY, msg[4], msg[5], msg[6], msg[7]);
		break;
		
	case WM_BOTTOMED:
		wind_set_int(msg[3], WF_BOTTOM, 0);
		break;
		
	case WM_SIZED:
	case WM_MOVED:
		wind_get(msg[3], WF_CURRXYWH, &cx, &cy, &cw, &ch);
		wind_set(msg[3], WF_CURRXYWH, msg[4], msg[5], msg[6], msg[7]);

		if (msg[0] == WM_SIZED)
			{
			wind_get(msg[3], WF_WORKXYWH, &wx, &wy, &ww, &wh);
			calc_viewlines(vinf, ww);
			
			if (vinf->top + wh / tx_height >= vinf->lines) 
				{
				vinf->top = vinf->lines - wh / tx_height;
				if (vinf->top < 0) vinf->top = 0;
				redraw_view(vinf, wx, wy, ww, wh);
				}
			else
				if (vinf->binmode && cw >= msg[6] && ch >= msg[7])
					redraw_view(vinf, wx, wy, ww, wh);
					
			set_viewslider(vinf);
			}
		break;

	case WM_FULLED:
		{
		int fx, fy, fw, fh;
		
		wind_get(msg[3], WF_CURRXYWH, &cx, &cy, &cw, &ch);
		wind_get(msg[3], WF_FULLXYWH, &fx, &fy, &fw, &fh);
		if (cx == fx && cy == fy && cw == fw && ch == fh)
			wind_get(msg[3], WF_PREVXYWH, &fx, &fy, &fw, &fh);
			
		wind_set(msg[3], WF_CURRXYWH, fx, fy, fw, fh);
		break;					
		}

	case WM_ARROWED:
		{
		switch(msg[4])
			{
			case WA_UPPAGE:
				if (vinf->top > 0) 
					{
					vinf->top -= wh / tx_height;
					if (vinf->top < 0) vinf->top = 0;
					redraw_view(vinf, wx, wy, ww, wh);
					set_viewslider(vinf);
					}
				break;
				
			case WA_DNPAGE:
				if (vinf->top + wh / tx_height < vinf->lines)
					{
					vinf->top += wh / tx_height;
					if (vinf->top + wh / tx_height >= vinf->lines) 
						{
						vinf->top = vinf->lines - wh / tx_height;
						if (vinf->top < 0) vinf->top = 0;
						}
						
					redraw_view(vinf, wx, wy, ww, wh);
					set_viewslider(vinf);
					}
				break;
				
			case WA_UPLINE:
				if (vinf->top > 0) 
					{
					vinf->top--;
					graf_mouse(M_OFF, 0L);
					blt_copy(NULL, wx, wy, NULL, wx, wy + tx_height, ww, wh - tx_height, S_ONLY);
					redraw_view(vinf, wx, wy, ww, tx_height);
					set_viewslider(vinf);
					graf_mouse(M_ON, 0L);
					}
				break;
				
			case WA_DNLINE:
				if (vinf->top + wh / tx_height < vinf->lines)
					{
					vinf->top++;
					graf_mouse(M_OFF, 0L);
					blt_copy(NULL, wx, wy + tx_height, NULL, wx, wy, ww, wh - tx_height, S_ONLY);
					redraw_view(vinf, wx, wy + wh - tx_height - (wh % tx_height), ww, tx_height + (wh % tx_height));
					set_viewslider(vinf);
					graf_mouse(M_ON, 0L);
					}
				break;
				
			case WA_LFPAGE:
				if (vinf->left > 0) 
					{
					vinf->left -= ww / tx_width;
					if (vinf->left < 0) vinf->left = 0;
					redraw_view(vinf, wx, wy, ww, wh);
					set_viewslider(vinf);
					}
				break;
				
			case WA_RTPAGE:
				if (vinf->left + ww / tx_width < vinf->width)
					{
					vinf->left += ww / tx_width;
					if (vinf->left + (ww / tx_width) >= vinf->width) 
						{
						vinf->left = vinf->width - ww / tx_width;
						if (vinf->left < 0) vinf->left = 0;
						}
						
					redraw_view(vinf, wx, wy, ww, wh);
					set_viewslider(vinf);
					}
				break;
				
			case WA_LFLINE:
				if (vinf->left > 0)
					{
					vinf->left--;
					graf_mouse(M_OFF, 0L);
					blt_copy(NULL, wx, wy, NULL, wx + tx_width, wy, ww - tx_width, wh, S_ONLY);
					redraw_view(vinf, wx, wy, tx_width, wh);
					set_viewslider(vinf);
					graf_mouse(M_ON, 0L);
					}
				break;
				
			case WA_RTLINE:
				if (vinf->width - vinf->left > ww / tx_width)
					{
					vinf->left++;
					graf_mouse(M_OFF, 0L);
					blt_copy(NULL, wx + tx_width, wy, NULL, wx, wy, ww - tx_width, wh, S_ONLY);
					redraw_view(vinf, wx + ww - tx_width - (ww % tx_width), wy, tx_width + (ww % tx_width), wh);
					set_viewslider(vinf);
					graf_mouse(M_ON, 0L);
					}
				break;
			}
		break;
		}
		
	case WM_HSLID:
	  vinf->left = (int)((long)msg[4] * (vinf->width - ww / tx_width) / 1000);
		redraw_view(vinf, wx, wy, ww, wh);
		set_viewslider(vinf);
		break;
		
	case WM_VSLID:
	  vinf->top = (long)msg[4] * (vinf->lines - wh / tx_height) / 1000;
		redraw_view(vinf, wx, wy, ww, wh);
		set_viewslider(vinf);
		break;
	}
}

/* ------------------------
   | Click in view window |
   ------------------------ */
void view_click(void)
{
long mark, start_mark;
int wx, wy, ww, wh, top, red;
int old_mx, old_my;
VIEWINFO *vinf;
int old_ln, new_ln, x_start, x_width;

DEBUG();
wind_get_int(ROOT, WF_TOP, &top);
vinf = get_viewinfo(top);
if (!vinf)	return;

wind_get(top, WF_WORKXYWH, &wx, &wy, &ww, &wh);

mark = get_mark();
if (mark != -1L)
	{
	if (evnt.kstate & 3)
		{
		if (mark < vinf->mark_start)
			vinf->mark_start = mark;
		else
			vinf->mark_end = mark + 1;
		redraw_view(vinf, wx, wy, ww, wh);
		}
	else
		{
		start_mark = vinf->mark_start = mark;
		vinf->mark_end   = mark + 1;
		redraw_view(vinf, wx, wy, ww, wh);
		
		old_mx = evnt.x;
		old_my = evnt.y;
		do
			{
			graf_mkstate(&evnt);

			if (evnt.x < wx + tx_width / 2 && vinf->left > 0)
				{
				old_mx += tx_width;
				vinf->left--;
				graf_mouse(M_OFF, 0L);
				blt_copy(NULL, wx, wy, NULL, wx + tx_width, wy, ww - tx_width, wh, S_ONLY);
				redraw_view(vinf, wx, wy, tx_width, wh);
				set_viewslider(vinf);
				graf_mouse(M_ON, 0L);
				}
			else
				if (evnt.x > wx + ww - tx_width / 2 && 
						vinf->width - vinf->left > ww / tx_width)
					{
					old_my -= tx_width;
					vinf->left++;
					graf_mouse(M_OFF, 0L);
					blt_copy(NULL, wx + tx_width, wy, NULL, wx, wy, ww - tx_width, wh, S_ONLY);
					redraw_view(vinf, wx + ww - tx_width - (ww % tx_width), wy, tx_width + (ww % tx_width), wh);
					set_viewslider(vinf);
					graf_mouse(M_ON, 0L);
					}

			if (evnt.y < wy + tx_height / 2 && vinf->top > 0)
				{
				old_my += tx_height;
				vinf->top--;
				graf_mouse(M_OFF, 0L);
				blt_copy(NULL, wx, wy, NULL, wx, wy + tx_height, ww, wh - tx_height, S_ONLY);
				redraw_view(vinf, wx, wy, ww, tx_height);
				set_viewslider(vinf);
				graf_mouse(M_ON, 0L);
				}
			else
				if (evnt.y > wy + wh - tx_height / 2 && 
						vinf->top + wh / tx_height < vinf->lines)
					{
					old_my -= tx_height;
					vinf->top++;
					graf_mouse(M_OFF, 0L);
					blt_copy(NULL, wx, wy + tx_height, NULL, wx, wy, ww, wh - tx_height, S_ONLY);
					redraw_view(vinf, wx, wy + wh - tx_height - (wh % tx_height), ww, tx_height + (wh % tx_height));
					set_viewslider(vinf);
					graf_mouse(M_ON, 0L);
					}
			
			mark = get_mark();
			red = FALSE;
			if (mark != -1)
				{
				if (mark <= start_mark)
					{
					if (vinf->mark_start != mark)
						{
						vinf->mark_start = mark;
						vinf->mark_end = start_mark + 1;
						red = TRUE;
						}
					}
				else
					{
					if (mark + 1 != vinf->mark_end)
						{
						vinf->mark_start = start_mark;
						vinf->mark_end = mark + 1;
						red = TRUE;
						}
					}

				if (red)
					{
					old_ln = (old_my - wy) / tx_height;
					new_ln = (evnt.y - wy) / tx_height;
	
					if (old_ln > new_ln)
						{
						x_start = 0;
						x_width = ww;
							
						old_ln++;	
						redraw_view(vinf, 
												wx + x_start, 
												wy + new_ln * tx_height,
											  x_width, 
											  (old_ln - new_ln) * tx_height);
						}
					else
						{
						if (old_ln == new_ln)
							{
							if (old_mx < evnt.x)
								{
								x_start = (old_mx - wx - 3) / tx_width;
								x_start *= tx_width;
								x_width = (evnt.x - old_mx + tx_width - 1) / tx_width + 1;
								x_width *= tx_width;
								}
							else
								{
								x_start = (evnt.x - wx - 3) / tx_width;
								x_start *= tx_width;
								x_width = (old_mx - evnt.x + tx_width - 1) / tx_width + 1;
								x_width *= tx_width;
								}
							}
						else
							{
							x_start = 0;
							x_width = ww;
							}
						
						new_ln++;	
						redraw_view(vinf, 
												wx + x_start, 
												wy + old_ln * tx_height,
											  x_width, 
											  (new_ln - old_ln) * tx_height);
						}
					}
				}

			old_mx = evnt.x;
			old_my = evnt.y;
			}while(evnt.bstate & 1);
			
		if (vinf->mark_start == vinf->mark_end)
			vinf->mark_start = vinf->mark_end = -1L;
		}
	}
set_view_menu();
}

/* ------------------
   | Calculate mark |
   ------------------ */
long get_mark(void)
{
int top, bytes_per_line;
int wx, wy, ww, wh;
long pos;
VIEWINFO *vinf;

DEBUG();
wind_get_int(ROOT, WF_TOP, &top);
vinf = get_viewinfo(top);

wind_get(top, WF_WORKXYWH, &wx, &wy, &ww, &wh);

if (vinf->binmode)
	{
	bytes_per_line = 0;
	while ((bytes_per_line * 4 + 7 + 2) * tx_width < ww)
		bytes_per_line++;
	bytes_per_line--;
	
	if (evnt.x - wx >= (bytes_per_line * 3 + 9) * tx_width &&
			evnt.x - wx <  (bytes_per_line * 4 + 9) * tx_width)
		{
		pos = vinf->top * bytes_per_line + 
					(evnt.y - wx) / tx_width +
					(evnt.y - wy) / tx_height * bytes_per_line;
		}
	else
		if (evnt.x - wx >= 7 * tx_width &&
				evnt.x - wx < (bytes_per_line * 3 + 7) * tx_width)
			{
			pos = vinf->top * bytes_per_line + 
						(evnt.y - wx - 7 * tx_width) / tx_width / 3 +
						(evnt.y - wy) / tx_height * bytes_per_line;
			}
		else
			return -1L;
	}
else
	{
	unsigned char *mem;
	long mem_len, pg;
	int xpos, mpos;
	
	mem = vinf->ptr;
	mem_len = vinf->size;
	pg = vinf->top + (evnt.y - wy) / tx_height;
	
	if (pg < 0) return -1L;
	
	while(pg--)
		{
		while(mem_len && *mem != '\n' && *mem != '\r')
			{
			mem++;
			mem_len--;
			}
	
		if (mem_len)
			{
			if (mem_len > 1 && *mem != mem[1] && (mem[1] == '\r' || mem[1] == '\n'))
				{
				mem++;
				mem_len--;
				}
		
			mem++;
			mem_len--;
			}
		}

	mpos = vinf->left + ((evnt.x - wx) / tx_width);
	if (*mem == 12)
		return mem - vinf->ptr;
		
	xpos = 0;
	while(mem_len && *mem != '\n' && *mem != 12 && *mem != '\r')
		{
		if (*mem != '\t')
			{
			if (xpos == mpos) return mem - vinf->ptr;
			mem++;
			xpos++;
			}
		else
			{
			do
				{
				xpos++;
				if (xpos == mpos) return mem - vinf->ptr;
				}while(xpos % ftp_opts.tab_size);
			mem++;
			}
			
		mem_len--;
		}
		
	return mem - vinf->ptr;
	}
	
if (pos > vinf->size || pos < 0)
	return -1L;
else
	return pos;
}

/* ---------------------
   | Set menu for view |
   --------------------- */
void set_view_menu(void)
{
int top;
VIEWINFO *vinf;

DEBUG();
wind_get_int(ROOT, WF_TOP, &top);

vinf = get_viewinfo(top);
if (!vinf) return;

menu_ienable(menu_adr, M_COPY, (vinf->mark_start > -1));
menu_icheck(menu_adr, M_VASCII, vinf->binmode == 0);
menu_icheck(menu_adr, M_VBIN, vinf->binmode == 1);
}

/* --------------------------
   | Set view mode to ASCII |
   -------------------------- */
void set_ascii_mode(void)
{
int top;
VIEWINFO *vinf;
int wx, wy, ww, wh;

DEBUG();
wind_get_int(ROOT, WF_TOP, &top);

vinf = get_viewinfo(top);
if (!vinf) return;

if (vinf->binmode)
	{
	wind_get(top, WF_WORKXYWH, &wx, &wy, &ww, &wh);
	vinf->binmode = 0;
	redraw_view(vinf, wx, wy, ww, wh);
	set_view_menu();

	calc_viewlines(vinf, ww);

	if (vinf->left + (ww / tx_width) >= vinf->width) 
		{
		vinf->left = vinf->width - ww / tx_width;
		if (vinf->left < 0) vinf->left = 0;
		}
	if (vinf->top + wh / tx_height >= vinf->lines) 
		{
		vinf->top = vinf->lines - wh / tx_height;
		if (vinf->top < 0) vinf->top = 0;
		}
		
	set_viewslider(vinf);
	}
}

/* ---------------------------
   | Set view mode to Binary |
   --------------------------- */
void set_binary_mode(void)
{
int top;
VIEWINFO *vinf;
int wx, wy, ww, wh;

DEBUG();
wind_get_int(ROOT, WF_TOP, &top);

vinf = get_viewinfo(top);
if (!vinf) return;

if (!vinf->binmode)
	{
	wind_get(top, WF_WORKXYWH, &wx, &wy, &ww, &wh);
	vinf->binmode = 1;
	redraw_view(vinf, wx, wy, ww, wh);
	set_view_menu();

	calc_viewlines(vinf, ww);

	if (vinf->left + (ww / tx_width) >= vinf->width) 
		{
		vinf->left = vinf->width - ww / tx_width;
		if (vinf->left < 0) vinf->left = 0;
		}
	if (vinf->top + wh / tx_height >= vinf->lines) 
		{
		vinf->top = vinf->lines - wh / tx_height;
		if (vinf->top < 0) vinf->top = 0;
		}
		
	set_viewslider(vinf);
	}
}

/* ---------------------
   | Copy to clipboard |
   --------------------- */
void view_copy(void)
{
#ifdef EVAL
	not_demo();
#else
	char scrap[256];
	int top;
	VIEWINFO *vinf;
	long fh, size;
	
DEBUG();
	wind_get_int(ROOT, WF_TOP, &top);
	
	vinf = get_viewinfo(top);
	if (!vinf) return;
	
	if (scrp_read(scrap))
		{
		if (!vinf->binmode)
			strcat(scrap, "SCRAP.TXT");
		else
			strcat(scrap, "SCRAP.BIN");
	
		fh = Fcreate(scrap, 0);
		if (fh > 0)
			{
			size = vinf->mark_end - vinf->mark_start;
			if (Fwrite((int)fh, size, vinf->ptr + vinf->mark_start) != size)
				form_alert(1, scrap_error);
			Fclose((int)fh);
			}
		else
			form_alert(1, scrap_error);
		}
#endif
}
